/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _BLUETOOTH_H
#define _BLUETOOTH_H

#include "../utils/xml/xml.h"

#define BLUETOOTH_BUFFERLEN 128

/** 
 * Provides Bluetooth communication channel.
 * @see CommLinkFactory
 * @see SerialPort
 */
class Bluetooth : public CommLink
{
public:
	// Registeration stuff.
	enum { ItemID = 'BluT' };
	const static wxString ItemDesc;


	Bluetooth();
	virtual ~Bluetooth();

	virtual wxInt8 Connect();
	virtual wxInt8 Disconnect();

	/** @param length Max length in current implementation is 255! */
	virtual wxInt8 SendBuffer(const wxByte *buffer, const wxUint16 length);
	virtual wxInt8 SendString(const wxString& string);
	
	/** @param length Number of bytes to be received. */
	virtual wxInt8 ReceiveBuffer(wxByte *buffer, const wxUint16 length);
	virtual wxInt8 ReceiveString(wxString& string);

	/** Perform an inquiry to get the module address of another nearby Bluetooth module. */
	wxInt8 Inquiry();

	/* ---------------- CONFIGURE ------------------ */

	/** Ask user for CommLink configuration. */
	virtual void Configure();

	/** XML stuff */
	virtual void					Load (XMLElement *myself, XMLLoader *loader);
	virtual XMLElement*		SaveSelf ();
	virtual wxString			GetXMLName ();
	static const wxString	GetShortXMLName() { return "commlink-bluetooth"; };

protected:
	wxInt8 ReceivePacket(); /**< Receive one packet and store it in the buffer. */

	SerialPort *mSerialPort; /**< SerialPort used to establish connection with Bluetooth host. */
	wxString	mPort;
	wxUint32	mBaudRate;

	wxByte mModuleAddress[6]; /**< Address of module to connect to. */
	wxByte mConnectionHandleLow; /** Low byte of connection handle. */
	wxByte mConnectionHandleHigh; /** High byte of connection handle. */

	wxByte mBuffer[BLUETOOTH_BUFFERLEN]; /**< Buffer used to send/receive packets. */
	bool mROK;
};

#endif /* _BLUETOOTH_H */
