/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _COMMLINK_H
#define _COMMLINK_H

#include "../utils/xml/xml.h"


/** 
 * Communication class, providing the communication channel between the
 * Robot class and the real robot.
 * @see Robot
 * @see CommLinkFactory
 */
class CommLink : public XMLSerializable
{
public:
	CommLink();
	virtual ~CommLink();

	/* ---------------- CONNECTION ------------------ */
	
	virtual wxInt8 Connect() =0;
	virtual wxInt8 Disconnect() =0;
	
	/* ---------------- I/O ------------------ */
	
	virtual wxInt8 SendBuffer(const wxByte *buffer, const wxUint16 length) =0;
	virtual wxInt8 SendString(const wxString& string) =0;
	virtual wxInt8 ReceiveBuffer(wxByte *buffer, const wxUint16 length) =0;
	virtual wxInt8 ReceiveString(wxString& string) =0;
	
	/** Use this method to resync the CommLink when synchronization is lost. */
	virtual wxInt8 Synchronize() { return 0; }
	
	/* ---------------- ACCESSORS ------------------ */

	bool GetConnected() { return mConnected; }

	/* ---------------- CONFIGURE ------------------ */

	/** Redefine this method to ask user for CommLink configuration. */
	virtual void Configure() {}

	/** --  XML stuff  -- */
	virtual void			Load (XMLElement *myself, XMLLoader *loader);
	virtual void			SaveChilds (XMLSaver *saver);
	virtual void			SetParentName (const wxString &parentName);
	virtual XMLElement*		SaveSelf ();
	static const wxString	GetShortXMLName() { return "commlink"; };
	
	
	/* ---------------- REGISTER STUFF ------------------ */
	void SetID(int id) {mID = id;}
	int GetID() {return mID;}

protected:
	bool mConnected;
	int mID;
};

#endif /* _COMMLINK_H */
