/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _COMMLINKFACTORY_H
#define _COMMLINKFACTORY_H

#include "../utils/factoryengine.h"
#include "../register.h"
#include "commlink.h"


/** Use this macro to register a new CommLink. */
#define RegisterCommLink(cl)	CommLinkFactory::mEngine.RegisterItem(cl::ItemID, &FactoryEngine<CommLink, int>::ItemCreator<cl>::CreateItem, cl::ItemDesc, cl::GetShortXMLName())


/** 
 * Factory for creating CommLinks.
 * @see CommLink
 */
class CommLinkFactory
{
public:

	/* ---------------- ACCESSORS -------------------- */

	/** Returns the number of CommLink types available. */
	static const wxUint32 GetNumTypes() { return mEngine.GetEntryNumber(); }

	/** Returns a description based on an index. */
	static const wxString GetIndexedDesc(wxUint32 index) {return mEngine.GetDescriptionByIndex(index); }

	/** Returns a description based on an ID. */
	static const wxString GetDesc(int id) {return mEngine.GetDescriptionByID(id);}
	
	

	/* ---------------- ID/INDEX CONVERSION -------------------- */

	/** Convert ID to index. */
	static wxUint32 GetIndexByID(int id);
	
	/** Convert index to ID. */
	static int GetIDByIndex(wxUint32 index) {return mEngine.GetIDByIndex(index);}



	/* ---------------- CREATION -------------------- */

	/** 
	 * Create a CommLink of a given id and return a pointer to it. The caller
	 * is responsible for destructing the CommLink.
	 */
	static CommLink* Create(int id);
	
	/** 
	 * Create a CommLink of a given index and return a pointer to it. The caller
	 * is responsible for destructing the CommLink.
	 */
	static CommLink* CreateByIndex(wxUint32 index);
	
	/** 
	 * Create a CommLink of a given XML name and return a pointer to it. The caller
	 * is responsible for destructing the CommLink.
	 */
	static CommLink* Create(const char* xmlName);

protected:

	friend class Register;

	/** The engine. */
	static FactoryEngine<CommLink, int>  mEngine;

};


#endif /* _COMMLINKFACTORY_H */
