/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _SERIALPORT_H
#define _SERIALPORT_H

#ifdef WIN32
	#include <windows.h>
	#include <wx/msw/winundef.h>
#endif

#include "../utils/xml/xml.h"
#include "commlink.h"

#define SERIAL_BUFFERLEN 512

/** 
 * Provides serial port communication channel. Very simple class, provides only what is necessary.
 * @see CommLink
 * @see CommLinkFactory
 */
class SerialPort : public CommLink
{
public:
	// register stuff
	enum { ItemID = 'Seri' };
	const static wxString ItemDesc;


	SerialPort();
	SerialPort(wxByte stopBits, wxUint32 baudRate);
	SerialPort(wxByte stopBits, wxUint32 baudRate, wxString	port);
	virtual ~SerialPort();

	virtual wxInt8 Connect();
	virtual wxInt8 Disconnect();
	virtual wxInt8 SendBuffer(const wxByte *buffer, const wxUint16 length);
	virtual wxInt8 SendString(const wxString& string);
	virtual wxInt8 ReceiveBuffer(wxByte *buffer, const wxUint16 length);
	virtual wxInt8 ReceiveString(wxString& string);

	/** Use this method to resync the CommLink when synchronization is lost. */
	virtual wxInt8 Synchronize();

	void SetBaudRate(wxUint32 baudRate) { mBaudRate = baudRate; }
	void SetCommPort(wxString commPort) { mPort = commPort; }

	/* ---------------- CONFIGURE ------------------ */
	/** Ask user for CommLink configuration. */
	virtual void Configure();

	/* ---------------- XML stuff ------------------ */
	virtual void					Load(XMLElement *myself, XMLLoader *loader);
	virtual XMLElement*		SaveSelf();
	virtual wxString			GetXMLName();
	static const wxString	GetShortXMLName() { return "commlink-serialport"; };

protected:
	char *mBuffer;

	wxString	mPort;
	wxUint32	mBaudRate;
	wxByte 		mParity, mStopBits, mByteSize;
#ifdef WIN32
	HANDLE 		mHandle;
	DCB				mDcb;
	COMMTIMEOUTS  mTimeOuts;
#else
	int				mFileDesc;
	FILE 			*mHandle;
#endif

};

#endif /* _SERIALPORT_H */
