/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "../status.h"
#include "../utils/iutils.h"
#include "../utils/xml/xml.h"
#include "commlink.h"

#include "webots.h"


const wxString Webots::ItemDesc = "Webots";

Webots::Webots() {
	mSocketClient = NULL;
}

Webots::~Webots() {
	if(mSocketClient) {
		mSocketClient->Destroy();	// call Close() automatically
		mSocketClient = NULL;
	}
}

wxInt8 Webots::Connect() {
	wxIPV4address addr;

	addr.LocalHost();
	addr.Service(10020);
	
	mSocketClient = new wxSocketClient(wxSOCKET_WAITALL | wxSOCKET_BLOCK);

	mSocketClient->Connect(addr, true);
	mConnected = mSocketClient->IsConnected();
	if(!mConnected)
		return 1;
	return 0;
}

wxInt8 Webots::Disconnect() {
	wxByte buffer[2];

	if(!mSocketClient) return 0;

	// Tell Webots to close the connection
	buffer[0]=0xFF;
	buffer[1]=0xFF;
	SendBuffer(buffer,2);

	mSocketClient->Destroy();	// call Close() automatically
	mSocketClient = NULL;
	mConnected = false;
	return 0;
}

wxInt8 Webots::SendBuffer(const wxByte *buffer, const wxUint16 length) {
	if(!mConnected) return 1;
	mSocketClient->Write(buffer, length);	// Write(const void * buffer, wxUint32 nbytes)
  if (mSocketClient->Error()) return 2;
	if (mSocketClient->LastCount()!=length) return 3;
	return 0;
}

wxInt8 Webots::SendString(const wxString& string) {
	if(!mConnected) return 1;
	
	return 1;
}

wxInt8 Webots::ReceiveBuffer(wxByte *buffer, const wxUint16 length) {
	if(!mConnected) return 1;
	mSocketClient->Read(buffer, length);
	if (mSocketClient->LastCount()!=length) return 3;
	if (mSocketClient->Error()) return 1;
	return 0;
}

wxInt8 Webots::ReceiveString(wxString& string) {
	if(!mConnected) return 1;
	
	return 1;
}

/* --- XML --- */
void Webots::Load (XMLElement *myself, XMLLoader *loader) {
}

XMLElement* Webots::SaveSelf () {	
	XMLElement *result = new XMLElement(GetShortXMLName().c_str());	
	return result;
}

wxString Webots::GetXMLName () {
	return GetShortXMLName();
}
