/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _WEBOTS_H
#define _WEBOTS_H

#include <wx/sckipc.h>

#include "../utils/xml/xml.h"
#include "commlink.h"

/** 
 * Provides TCP/IP communication channel to Webots simulator.
 * @see CommLink
 * @see CommLinkFactory
 */
class Webots : public CommLink
{
public:

	// register stuff
	enum { ItemID = 'Webo' };
	const static wxString ItemDesc;


	Webots();
	virtual ~Webots();

	/**
	* Connect to Webots on localhost. Wait until connection completed.
	* @return Error code, 0 if ok, 1 if cannot connect.
	*/
	virtual wxInt8 Connect();

	virtual wxInt8 Disconnect();

	/**
	* Send data to Webots on localhost
	* @return Error code, 0 if ok, 1 if a socket error occur,
	* 2 if the number of sent bytes is different from length.
	*/
	virtual wxInt8 SendBuffer(const wxByte *buffer, const wxUint16 length);

	/**
	* Not yet implemented.
	* @return Error code 1.
	*/
	virtual wxInt8 SendString(const wxString& string);

	/**
	* Read data from Webots on localhost
	* @return Error code, 0 if ok, 1 if a socket error occur.
	*/
	virtual wxInt8 ReceiveBuffer(wxByte *buffer, const wxUint16 length);

	/**
	* Not yet implemented.
	* @return Error code 1.
	*/
	virtual wxInt8 ReceiveString(wxString& string);

	/* XML stuff */
	virtual void			Load (XMLElement *myself, XMLLoader *loader);
	virtual XMLElement*		SaveSelf ();
	virtual wxString		GetXMLName ();
	static const wxString	GetShortXMLName() { return "commlink-webots"; };

protected:
	wxSocketClient *mSocketClient;
};

#endif /* _WEBOTS_H */
