/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _EVOVIEW_H
#define _EVOVIEW_H

class EvoModel;
class ModelSettings;
class Controller;

/**
 * Displays evolution status and statistics, sensor data and neural activity as
 * observed in EvoModel.
 * @see EvoModel
 */
class EvoView : public wxScrolledWindow
{
public:
	/** Constructor */
	EvoView(Controller *controller, EvoModel *evoModel);

	/** Destructor */
	virtual ~EvoView();

	/* ------------------------ EVENTS ------------------ */

	/** Repaint. */
	void OnUpdateView(wxCommandEvent& event);

	/* Paint event handler. */
	void OnPaint(wxPaintEvent& event);

	/** Size event handler. */
	void OnSize(wxSizeEvent& event);

	/** Mouse left click handler. */
	void OnLeftClick(wxMouseEvent& event);

	/** Mouse right click handler. */
	void OnRightClick(wxMouseEvent& event);
	
	/** Disable to prevent drawing of view. */
	void SetViewEnabled(bool enable) { mEnabled = enable; }

protected:
	DECLARE_EVENT_TABLE()

	/* ------------------------ PAINTING ------------------ */

	/** Return the size required to draw everything (according to mMode */	
    wxSize EvoView::GetPaintSize( void ) ;

	/** Actually paints the view. */
	void Paint(wxDC& dc);

	/** Paints the Robot. */
	void PaintRobot(wxDC& dc);

	/** Paints the NeuralNetwork. */
	void PaintNeuralNetwork(wxDC& dc);

	/** Paints the Experiment. */
	void PaintExperiment(wxDC& dc);

	/** Paints the EvoModel. */
	void PaintEvoModel(wxDC& dc);

	/* ----------------------- VARIABLES ------------------- */

	EvoModel *mEvoModel;
	ModelSettings *mModelSettings;

	static const wxUint8 NumModes; /**< Number of view modes. */
	wxUint8 mMode; /**< Current view mode. */
	wxString mViewString; /**< String describing current view. */

	wxSize mSize; /**< Size of view. */
	bool mEnabled; /**< Don't draw view if false. */
};

#endif /* _EVOVIEW_H */
