/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "../status.h"
#include "../robot/dummy.h"
#include "../nn/neuralnetwork.h"
#include "../utils/iutils.h"
#include "../utils/xml/xml.h"
#include "../utils/randomtools.h"

#include "dummyexperiment.h"

const wxString DummyExperiment::ItemDesc = "Dummy";

DummyExperiment::DummyExperiment() {

	mNeuronInput = new float[8];
	mMotorScale = 50;
	mMotorSpeeds[0] = mMotorSpeeds[1] = 0.0f;
	mMotorSpeeds[2] = 1.0f;
}

DummyExperiment::~DummyExperiment() {

}

wxInt8 DummyExperiment::InitRobot() {
	mDummy = (Dummy *)mRobot;
	return 0;
}

wxInt8 DummyExperiment::InitEpoch(wxUint16 cycle) {
	wxInt8 flagEpoch=0;//1 continue InitEpoch & 0 stop InitEpoch
 		
	return (flagEpoch);
}

wxInt8 DummyExperiment::SetNNInput() {
	mDummy->ReadSensors();
	for(wxUint8 i=0; i<8; i++)
		mNeuronInput[i] = RandomTools::RandUnityDouble();
	mNeuralNetwork->SetInput(mNeuronInput);
	return 0;
}

wxInt8 DummyExperiment::ProcessNNOutput() {
	float *output = mNeuralNetwork->GetOutputValues();
	mMotorSpeeds[0] = output[0] - output[1];
	mMotorSpeeds[1] = output[2] - output[3];
	return mDummy->SetMotors(mMotorSpeeds);
}

float DummyExperiment::ComputeFitness() {

	return 0.0f;
}

/* --- XML --- */
void DummyExperiment::Load (XMLElement *myself, XMLLoader *loader) {
	mMotorScale = myself->GetAttribute("motor_scale")->GetByteValue();
}

XMLElement* DummyExperiment::SaveSelf () {
	XMLElement *result = new XMLElement(GetShortXMLName().c_str());	
	result->AddAttribute(new XMLAttribute("motor_scale",mMotorScale));
	return result;
}

wxString DummyExperiment::GetShortXMLName() { return DummyExperiment::GetStaticXMLType(); }
