/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "../nn/neuralnetwork.h"
#include "../utils/xml/xml.h"

#include "experiment.h"

Experiment::Experiment() {
	
	mNeuralNetwork = NULL;
	mNeuronInput = NULL;
}

Experiment::~Experiment() {
	if(mNeuronInput)
		delete [] mNeuronInput;
}

void Experiment::StartLog(Logger *logger) {
	logger->WriteString("%EXPERIMENT - " + GetDesc() + "\n");
	LogLegenda(logger);
}

void Experiment::EndLog(Logger *logger) { }

void Experiment::Paint(wxDC& dc, wxPoint& position, wxSize& size) {
	/* Draw experiment description */
	wxFont font = dc.GetFont();
	font.SetWeight(wxBOLD);
	dc.SetFont(font);
	dc.DrawText(wxString::Format("Experiment: %s", GetDesc().c_str()), position.x + 10, position.y + 15);
	font.SetWeight(wxNORMAL);
	dc.SetFont(font);
	
	/* Draw experiment */
	position.x += 10;
	position.y += 50;
	size.SetHeight(size.GetHeight() - 35);
	size.SetWidth(size.GetWidth() - 20);
	PaintExperiment(dc, position, size);
}

/* --- XML --- */
void Experiment::Load (XMLElement *myself, XMLLoader *loader) {
}

XMLElement* Experiment::SaveSelf () {
	XMLElement *result = new XMLElement("experiment");
	return result;
}

void Experiment::SaveChilds (XMLSaver *saver) {

}

void Experiment::SetParentName (const wxString &parentName) {

}

wxString Experiment::GetXMLName () { return GetShortXMLName(); }
wxString Experiment::GetShortXMLName() { return Experiment::GetStaticXMLType(); }
