/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "../status.h"
#include "experiment.h"

#include "experimentfactory.h"




DualIDFactoryEngine<Experiment> ExperimentFactory::mEngine;


wxUint32 ExperimentFactory::GetIndexByID(DualID id)
{
	wxUint32 index = 0;
	mEngine.GetSubIndexByID(id, index);
	return index;	
}


Experiment* ExperimentFactory::Create(int id, int robotID)
{
	Experiment* exp = mEngine.CreateItemWithID(std::make_pair(id, robotID));
	if(exp != NULL)
		exp->SetID(std::make_pair(id, robotID));
	return exp;
}

	
Experiment* ExperimentFactory::CreateByIndex(wxUint32 index, int robotID)
{
	Experiment* exp = mEngine.CreateItemWithSubIndex(index, robotID);
	if(exp != NULL)
		exp->SetID(mEngine.GetIDBySubIndex(index, robotID));
	return exp;
}

	
Experiment* ExperimentFactory::Create(const char* xmlName, int robotID)
{
	// This is not so clean.
	// In the current implementation, the engine will return the first
	// registered experiment with such xml name. If the exp is registered
	// many times for many robots, the first one registered will be used.
	// It shouldn't even be a problem, but it is not very clean and my
	// harm in (unlikely) futur changes.
	
	DualID id;
	Experiment* exp = NULL;
	if(mEngine.GetIDByXMLName(xmlName, id))
		exp = mEngine.CreateItemWithID(id);
	if(exp != NULL)
		exp->SetID(id);
	return exp;
}
