/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _EXPERIMENTFACTORY_H
#define _EXPERIMENTFACTORY_H

#include "../utils/factoryengine.h"


class Experiment;


/** Use this macro to register a new experiment. */
#define RegisterExperiment(exp,rob) ExperimentFactory::mEngine.RegisterItem(std::make_pair((int)exp::ItemID, (int)rob::ItemID), &DualIDFactoryEngine<Experiment>::ItemCreator<exp>::CreateItem, exp::ItemDesc, exp::GetStaticXMLType())


/**
 * Factory for creating Experiments.
 * @see FactoryEngine
 */
class ExperimentFactory
{
public:

	/* ---------------- ACCESSORS -------------------- */

	/** Returns the number of experiments associated to a particular robot. */
	static const wxUint32 GetNumTypes(int robotID) {return mEngine.GetEntryNumber(robotID);}
	
	/** Returns a description based on an index and robot id. */
	static const wxString GetIndexedDesc(wxUint32 index, int robotID) {return mEngine.GetDescriptionBySubIndex(index, robotID);}

	/** Returns a description based on an experiment ID and robot ID. */
	static const wxString GetDesc(int id, int robotID) {return mEngine.GetDescriptionByID(std::make_pair(id, robotID));}
	
	/** Converts an experiment dual index (a std::pair<int, int> as (expID, robotID) into an index. */
	static wxUint32 GetIndexByID(DualID id);
	

	/* ---------------- CREATION -------------------- */
	
	/** Create an experiment based on an id. */
	static Experiment* Create(int id, int robotID);
	
	/** Create an experiment based on an index. */
	static Experiment* CreateByIndex(wxUint32 index, int robotID);
	
	/** Create an experiment based on a xml name. */
	static Experiment* Create(const char* xmlName, int robotID);

protected:


	friend class Register;
	
	/** The engine. */
	static DualIDFactoryEngine<Experiment> mEngine;


};

#endif /* _EXPERIMENTFACTORY_H */
