/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _KOALAEXPERIMENT_H
#define _KOALAEXPERIMENT_H

#include "../utils/xml/xml.h"
#include "../nn/neuralnetwork.h"

#include "experiment.h"

/** 
 * Default Experiment for Koala.
 * @see Experiment
 */
class KoalaExperiment : public Experiment
{
public:
	// Registeration stuff.
	enum { ItemID = 'KoEx' };
	const static wxString ItemDesc;
	
	/** Constructor */
	KoalaExperiment();
 
	/** Destructor */
	virtual ~KoalaExperiment();
	
	/**
	 * Method that returns the number of outputs for the neural network.
	 * @return The number of output neurons.
	 */
	virtual wxUint8 GetNumOutputNeurons() { return 4; }

	/**
	 * Method that returns the number of inputs for the neural network
	 * @return the number of sensors
	 */
	virtual wxUint8 GetNumSensorNeurons() { return 4; }

	/** Initialize the robot, called when new connection with robot has been established. */
	virtual wxInt8 InitRobot() { return 0; }

	/** Initialize an epoch. */
	virtual wxInt8 InitEpoch(wxUint16 cycle) { return 0; }

	/**
	 * Method that gets the sensors values of the robot and sets them to the neural network input.
	 * @return Error code, zero if ok.
	 */
	virtual wxInt8 SetNNInput() { return 0; }

	/**
	 * Method that gets the ouputs of the neural network and sets them to the robot
	 * @return Error code, zero if ok.
	 */
	virtual wxInt8 ProcessNNOutput() { return 0; }

	/**
	 * Comput the fitness value of the current individual at this very moment.
	 * @return The fitness value.
	 */
	virtual float ComputeFitness() { return 0.0f; }

	/* ---------------- XML -------------------- */
	virtual void			Load (XMLElement *myself, XMLLoader *loader);
	virtual XMLElement*		SaveSelf ();
	virtual wxString		GetShortXMLName();
	static wxString			GetStaticXMLType() { return "experiment-koala"; }
};

#endif /* _KOALAEXPERIMENT_H */
