/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

#ifndef _TIMESERIESEXPERIMENT_H
#define _TIMESERIESEXPERIMENT_H

#include "experiment.h"

class Dummy;
class EvoModel;

WX_DECLARE_LIST(wxPoint, PointListT);

/**	experiment for time series analysis **************/
/*
/*	See "docs/experiments/" for more documentation
/*
/*****************************************************/

class TimeSeriesExperiment : public Experiment
{
public:
	TimeSeriesExperiment();
	virtual ~TimeSeriesExperiment();

	// Registeration stuff.
	enum { ItemID = 'TmSe' };
	const static wxString ItemDesc;


	/* ------------------------------- VIRTUAL -------------------- */

	/**
	 * Method that returns the number of inputs for the neural network
	 * @return the number of sensors
	 */
	virtual wxUint8 GetNumSensorNeurons() { return mNumInput; }

	/**
	 * Method that returns the number of outputs for the neural network.
	 * @return The number of output neurons.
	 */
	virtual wxUint8 GetNumOutputNeurons() { return mNumOutput; }

	/** Initialize an epoch. */
	//virtual wxInt8 InitEpoch();
	virtual wxInt8 InitEpoch(wxUint16 cycle);

	/**Initialize the (dummy) robot*/
	virtual wxInt8 InitRobot();

	/** Save an epoch. */
	virtual wxInt8 SaveEpoch(wxString &filename);

	/**
	 * Method that gets the sensors values of the robot and sets them to the neural network input.
	 * @return Error code, zero if ok.
	 */
	virtual wxInt8 SetNNInput();

	/**
	 * Method that gets the ouputs of the neural network.
	 * @return Error code, zero if ok.
	 */
	virtual wxInt8 ProcessNNOutput();

	/**
	 * Comput the fitness value of the current individual at this very moment.
	 * @return The fitness value, preferably between 0.0 and 1.0.
	 */
	virtual float ComputeFitness();//{ return mFitness; }
	
	/**Configure file**/
	virtual void Configure();

	/** Loas datas from file**/
	void LoadFromDisk(const wxString &filename, bool xmlTag=false);

	void PaintExperiment(wxDC& dc, wxPoint& position, wxSize& size) ;




		/* ---------------- XML -------------------- */
	virtual void			Load (XMLElement *myself, XMLLoader *loader);
	virtual XMLElement*		SaveSelf ();	
	virtual wxString		GetShortXMLName();
	static wxString			GetStaticXMLType() { return "experiment-timeseriesexperiment"; }

	
	/* ---------------- LOGGING -------------------- */

	/** Write 'legenda' to log file. */
	virtual void LogLegenda(Logger *logger);

	/** Called by logger when new data should be logged. */
	virtual void Log(Logger *logger);

protected:
	wxUint8 mNumInput, mNumOutput;
	wxUint16 mSetLength, mCurIndex;
	float **mInput, **mOutput, **mTarget;
	Dummy *mDummy;
	wxString mFileName;
	EvoModel *mEvoModel;

	PointListT *mOutputPoints;
	PointListT *mTargetPoints;

};

#endif /* _TIMESERIESEXPERIMENT_H */
