/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "../utils/randomtools.h"
#include "genome.h"

#include "crossover.h"

Crossover::Crossover() {
	mRate = 0.1f;
}

Crossover::Crossover(float rate) {
	mRate = rate;
}

Crossover::~Crossover() {
	
}

wxInt8 Crossover::Apply(Genome *g1, Genome *g2) {
	wxUint16 genomeLength, crosspoint;
	wxUint8 tmp;

	// decide if there is a crossover
	if(RandomTools::Flip(mRate)) {
		genomeLength = g1->GetLength();
		
		// choose randomly the crosspoint
		crosspoint = RandomTools::RandInt16(genomeLength);
		
		// for each bit after the cross point swap the bits of the two genomes
		for(wxUint16 i=crosspoint; i<genomeLength; i++) {
			tmp = g1->GetValue(i);
			g1->SetValue(i, g2->GetValue(i));
			g2->SetValue(i, tmp);
		}
	}
	return 0;
}
