/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _CROSSOVER_H
#define _CROSSOVER_H

#include "igenoperator.h"

class Genome;

/**
 * Crossover genetic operator. Assumes equal length genomes consisting of wxUint8's.
 * @see Genome
 */

class Crossover : public IGenOperator
{
public:
	/** Default constructor. */
	Crossover();

	/** Constructs Crossover operator with given crossover rate. */
	Crossover(float rate);
	
	/** Destructor. */
	virtual ~Crossover();

	/**
	 * Method which applies a crossover to two genomes with mRate probability
	 * @param g1 is the first genome
	 * @param g2 is the second genome
	 */
	wxInt8 Apply(Genome* g1, Genome* g2);

	virtual GenOperatorType GetType() { return CROSSOVER; }

	/**
	 * Accessor method.
	 * @param rate is the probabilty for having a crossover
	 */
	void SetRate(float rate) { mRate = rate; }

	/**
	 * Accessor method.
	 * @return Returns the rate of crossover.
	 */
	float GetRate() { return mRate; }

protected:
	float mRate; /**< Chance of applying crossover. */
};

#endif /* _CROSSOVER_H */
