/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "../utils/randomtools.h"
#include "genome.h"

#include "mutation.h"

Mutation::Mutation() {
	mRate = 0.05f;
}

Mutation::Mutation(float rate) {
	mRate = rate;
}

Mutation::~Mutation() {
	
}

wxInt8 Mutation::Apply(Genome *g1, Genome *g2) {
	wxUint16 genomeLength = g1->GetLength();
	
	//for each bit of the genomes
	for(wxUint16 i=0; i<genomeLength; i++) {
		if(RandomTools::Flip(mRate))
			g1->SetValue(i, !(g1->GetValue(i)));
		if(RandomTools::Flip(mRate))
			g2->SetValue(i, !(g2->GetValue(i)));
	}
	return 0;
}
