/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>
#include <math.h>

#include "../status.h"
#include "../utils/randomtools.h"
#include "genome.h"
#include "individual.h"
#include "population.h"

#include "truncation.h"


Truncation::Truncation() {
	mNumChildren = 4;
}

Truncation::Truncation(wxUint16 numChildren) {
	mNumChildren = numChildren;
}

Truncation::~Truncation() {

}

Population *Truncation::Reproduce(Population *population) {
	float fitness, maxFitness;
	Individual *individual;
	wxUint16 size = population->GetSize(), numParents = (wxUint16)(ceil((double)size / mNumChildren));
	wxUint16 swap1;
	
	Population *newPopulation = new Population(size, population->GetCurGeneration() + 1);

	// Reproduce top ranked individuals
	for(wxUint16 i=0; i < numParents; i++) {
		maxFitness = population->GetIndividual(i)->GetFitness();

		for(wxUint16 j=i+1; j<size; j++) {
			fitness = population->GetIndividual(j)->GetFitness();

			//if this individual is better than the previous one
			if(fitness > maxFitness) {
				//swap individual i and individual j
				individual = population->GetIndividual(i);
				population->SetIndividual(i, population->GetIndividual(j));
				population->SetIndividual(j, individual);
				
				//set the new bestFitness
				maxFitness = fitness;
			}
		}
		for(wxUint16 j=0; j < mNumChildren && (i * mNumChildren + j) < size; j++)
			newPopulation->SetIndividual(i * mNumChildren + j, new Individual(new Genome(population->GetIndividual(i)->GetGenome(),population->GetCurGeneration(),i * mNumChildren + j)));
	}

	// shuffle the new population, swap individuals randomly chosen
	for(wxUint16 i=0; i<size; i++) {
		swap1 = RandomTools::RandInt16(size);
		individual = newPopulation->GetIndividual(swap1);
		newPopulation->SetIndividual(swap1, newPopulation->GetIndividual(i));
		newPopulation->SetIndividual(i, individual);
	}

	// return the new population
	return newPopulation;
}
