/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _TRUNCATION_H
#define _TRUNCATION_H

#include "ireproduce.h"
class Population;

/**
 * Reproduction using truncation selection.
 * @see IReproduce
 * @see Population
 */
class Truncation : public IReproduce
{
public:

	/** Construct default Truncation. */
	Truncation();

	/** Construct with the number of children */
	Truncation(wxUint16 numChildren);

	/** Destructor. */
	virtual ~Truncation();

	/**
	 * Reproduce the population.
	 * @param population is the population which must be reproduced
	 * @return the new population
	 */
	Population* Reproduce(Population* population);

	/**
	 * Accessor method
	 * @param numChildren is the number of children for the new population
	 */
	void SetNumChildren(wxUint16 numChildren) { mNumChildren = numChildren; }

	/**
	 * Accessor method
	 * @return the number of children for the new population
	 */
	wxUint16 GetNumChildren() { return mNumChildren; }

protected:
	/** The number of children for the new population, it must be divisor of the population size. */
	wxUint16 mNumChildren;
};

#endif /* _TRUNCATION_H */
