/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _MANUALMODEL_H
#define _MANUALMODEL_H

#include <wx/thread.h>

#include "utils/observable.h"
class ModelSettings;
class wxJoystick;
class jsJoystick;

/** 
 * Used to contain and control an instance of Robot when manual 
 * (joystick) control is chosen.
 * @see ManualView
 * @see Robot
 */
class ManualModel : public Observable, wxThread
{
public:
	/** Construct a default ManualModel. */
	ManualModel(ModelSettings *modelSettings);

	/** Destroy this ManualModel. */
	~ManualModel();
	
	/** Start manual control of robot. */
	wxInt8 Start();
	/** Stop manual control of robot. */
	wxInt8 Stop();

	/** Entry point of the thread, called by wxWindows. Implements control loop. */
	ExitCode Entry();
	
	/** Returns current Robot. */
	Robot* GetRobot() { return mRobot; }
	
	/** Paint the model. */
	void Paint(wxDC& dc, wxPoint& position, wxSize& size);
	
protected:
	ModelSettings *mModelSettings; /**< Pointer to ModelSettings. */
	Robot *mRobot; /**< Pointer to Robot object that is controlled. */
	jsJoystick *mJoystick; /**< Joystick object used for reading joystick values. */

	bool mLedState; /**< LED - currently on or off. */
	float mMotorSpeeds[3]; /**< Array to maintain motor speeds (between -1.0 and 1.0). */
	float mRollAxis, mPitchAxis, mYawAxis, mThrottleAxis;
	wxUint16 mLoopDelay; /**< Delay (in ms) that should be waited at the end of each loop. */
};

#endif /* _MANUALMODEL_H */
