/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "controller.h"
#include "manualmodel.h"
#include "robot/robot.h"

#include "manualview.h"

ManualView::ManualView(Controller *controller, ManualModel *manualModel) : wxWindow(controller, -1, wxPoint(0,0)) {
	mManualModel = manualModel;
	mRobot = mManualModel->GetRobot();

	SetBackgroundColour(wxColour(255,255,255));
	SetForegroundColour(wxColour(0,0,255));

	mSize = GetSize();
}

ManualView::~ManualView() {

}

/* --------------------- EVENTS ---------------- */

BEGIN_EVENT_TABLE(ManualView, wxWindow)
	EVT_MENU(UPDATE_VIEW, ManualView::OnUpdateView)
	EVT_PAINT(ManualView::OnPaint)
	EVT_SIZE(ManualView::OnSize)
END_EVENT_TABLE()


void ManualView::OnPaint(wxPaintEvent& event) {
	wxPaintDC dc(this);
	Paint(dc);
}

void ManualView::OnSize(wxSizeEvent& event) {
	mSize = GetSize();
	wxClientDC dc(this);
	Paint(dc);
}

/* -------------------- UPDATE ------------------ */

void ManualView::Update(Observable* observable, void* argument) {
	wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED, UPDATE_VIEW);
	wxPostEvent(this, event);
}

void ManualView::OnUpdateView(wxCommandEvent& event) {
	wxClientDC dc(this);
	Paint(dc);
}

/* ---------------- PAINT METHODS ------------------*/

void ManualView::Paint(wxDC& dc) {
	/* Initialize buffered image. */
	wxBitmap* bmpBlit = new wxBitmap(mSize.GetWidth(), mSize.GetHeight());
	wxMemoryDC* memDC= new wxMemoryDC();
	memDC->SelectObject(*bmpBlit);
	memDC->BeginDrawing();
	memDC->SetBackground(*wxWHITE_BRUSH);
	memDC->Clear();
	memDC->SetBackground(wxNullBrush);

	memDC->SetTextForeground(*wxBLACK);
	memDC->SetTextBackground(*wxWHITE);

	wxPoint position;
	wxSize size;
	int boundary = (int)(0.6f * mSize.GetWidth());

	/* Set font size. */
	wxFont font = memDC->GetFont();
	font.SetPointSize(8);
	memDC->SetFont(font);

	memDC->DrawText("1/1", mSize.GetWidth() - 25, 5);

	/* Set font size. */
	font.SetPointSize(10);
	memDC->SetFont(font);

	/* Draw Robot. */
	position.x = 0;
	position.y = 0;
	size.SetWidth(boundary);
	size.SetHeight(mSize.GetHeight());
	memDC->SetClippingRegion(position.x, position.y, size.GetWidth(), size.GetHeight());
	mRobot->Paint(*memDC, position, size);
	memDC->DestroyClippingRegion();

	/* Draw ManualModel. */
	position.y = 0;
	position.x = boundary;
	size.SetHeight(mSize.GetHeight());
	size.SetWidth(mSize.GetWidth() - boundary);
	memDC->SetClippingRegion(position.x, position.y, size.GetWidth(), size.GetHeight());
	mManualModel->Paint(*memDC, position, size);
	memDC->DestroyClippingRegion();

	memDC->SetBrush(wxNullBrush);
	memDC->SetPen(wxNullPen);

	memDC->EndDrawing();
	dc.Blit(0, 0, mSize.GetWidth(), mSize.GetHeight(), memDC, 0, 0, wxCOPY);
	delete bmpBlit;
	delete memDC;
}
