/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _MANUALVIEW_H
#define _MANUALVIEW_H

#include "utils/observable.h"
class Controller;
class ManualModel;
class Robot;

/** 
 * Used in combination with ManualModel to show Robot data when manual controlled.
 * @see ManualModel
 * @see Robot
 */
class ManualView : public wxWindow, public IObserver
{
public:
	/** Construct default view for ManualModel. */
	ManualView(Controller *controller, ManualModel *manualModel);
	
	/** Destructor. */
	~ManualView();

	void Update(Observable* observable, void* argument);
	void OnUpdateView(wxCommandEvent& event);

	/** Paint event handler. */
	void OnPaint(wxPaintEvent& event);
	/** Size event handler. */
	void OnSize(wxSizeEvent& event);

protected:
	DECLARE_EVENT_TABLE()

	/** Paint the view. */
	void Paint(wxDC& dc);
	
	/* Member variables */
	ManualModel *mManualModel;
	Robot *mRobot;
	wxSize mSize;
};

#endif /* _MANUALVIEW_H */
