%--------------------------------------------------------------------- 
%   Autonomous System Laboratory (LSA2), EPFL, Lausanne, Switzerland
%	Adaptive Intelligence Laboratory (AI Lab), University Utrecht 
%---------------------------------------------------------------------
%
%FUNCTION analyse_evolution
%
%   Run through directories see ANALYSE_TEMPLATE and calculate and save
%   some calculated parameters about evolved paramteres for every
%   generation in the file "evolution.mat". The parameters can be plotted
%   with PLOT_EVOLUTION
%
%   It assumes that when you run the function you are in the "..\genome" directory
%
%    user=1 if present, i.e. if the function might ask the user some
%   paramters
%
%   It assumes also the following log structure:
%       ..\genome\population_2\genome_2_45
%
%       Where 2 is the number of the population and 45 is the number of the
%       individual.
%
%       "..\genome" is a directory where population are stored
%
%       "population_2" is a folder containing all files for generation number 2
%       datas
%
%       "genome_2_45" is the file containing evolution datas for individual
%       45, generation 2. Usually it can be read using the function "load_log"
%
%   If you want to calculate other quantities of interest, follow the
%   template
%
%   see also ANALYSE, PLOT_EVOLUTION
function analyse_evolution(user)

if (nargin==0)
    user=1;
end

%index to run through all files
indPop=0;
indIndi=0;
%number of
numPop=0; %population 
numIndi=0; %inddividual
%found in the directory


cd genome;
%load the file "evolution.mat"
if (exist('evolution.mat')==2 & user==1)
    reply = input('''evolution.mat'' already exist, do you want to overwrite Y/N \n','s');
    if isempty(reply)
        reply ='Y' ;
    end
    if (strcmp(reply,'Y')  || strcmp(reply,'y'))
        
        delete evolution.mat
    else
        return;
    end
end

%get input from the user, if present.
d=dir('');
[s1, s2]=size(d);
if (user==1)
    maxNumPop = input('What is the maximum number of population? over which you want to proceed the EVOLUTION analysis\n');
    if isempty(maxNumPop)
        maxNumPop =100000 ;
    end
else
    maxNumPop =100000 ;
end

%run through the directory to find the directory of every population   
for i=1:s1 
    test=0;
    name=['population_',num2str(indPop)];
    eval(['if (exist (''', name,''',''dir'')==7) test=1; end'])
    if (test==1)  %the directory exist
        numPop=numPop+1;
        eval(['cd ', 'population_', num2str(indPop) ])
        d2=dir('*.log');
        [ss1, ss2]=size(d2);
        indIndi=0;
        numIndi=0;
        
        while numIndi < ss1
            test2=0;
            name2=['genome_', num2str(indPop),'_',num2str(indIndi),'.log'];
            eval(['if (exist( ','''',name2, '''',')==2) test2=1; end'])
            
            if (test2==1) %the file exist
                numIndi=numIndi+1;
                %load evolution datas
                eval(['load_log(','''',name2,'''',')'])
                %%%%%%%%%%%%%%%%Store the data from file for further calculation%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %neuron
                if (exist('Sign','var'))
                    SignStore(:,numIndi)=Sign;
                end
                if (exist('Threshold','var'))
                    ThresholdStore(:,numIndi)=Threshold;
                end
                if (exist('MembraneTimeC','var'))
                    MembraneTimeCStore(:,numIndi)=MembraneTimeC;
                end
                %synapse
                if (exist('Existence','var')) % if the variable was evoluated
                    ExistenceStore(:,:,numIndi)=Existence;
                end
                if (exist('Strength','var')) % if the variable was evoluated
                    if (~exist('Existence','var'))
                        StrengthStore(:,:,numIndi)=Strength;
                    else
%                        StrengthStore(:,:,numIndi)=Strength.*Existence;
                          StrengthStore(:,:,numIndi)=Strength;
                    end
                end
                if (exist('Delay','var')) % if the variable was evoluated
                    if (~exist('Existence','var'))
                        DelayStore(:,:,numIndi)=Delay;
                    else
                     %   DelayStore(:,:,numIndi)=Delay.*Existence;
                      DelayStore(:,:,numIndi)=Delay;
                    end
                end
                if (exist('TimeC','var')) % if the variable was evoluated
                    if (~exist('Existence','var'))
                        TimeCStore(:,:,numIndi)=TimeC;
                    else
                   %     TimeCStore(:,:,numIndi)=TimeC.*Existence;
                         TimeCStore(:,:,numIndi)=TimeC;
                    end
                end
                %mixed
                if (0)
                   if (exist('Existence','var') & exist('Sign','var')) 
                     for k=1:4
                          tmp(:,k)=Sign.*Existence(:,k);
                      end
                     
                     ExistenceSignStore(:,:,numIndi)=tmp;
                   end 
                end
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                fclose('all'); 
                
            else
                display(['could not open file ', name2])
            end         
            indIndi=indIndi+1;
        end
        nameSave='';
        %%%%%%%%%%%%Calculate some variable with the sored parameters%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %neuron
        if (exist('SignStore','var'))
            SignMoy(:,numPop)=mean(SignStore,2);
            SignStd(:,numPop)=std(SignStore,1,2);
            SignStdMoy(numPop)=mean(SignStd(:,numPop));
            SignPopMoy(numPop)=mean(SignMoy(:,numPop));
            nameSave=[nameSave, ' SignMoy SignPopMoy SignStd  SignStdMoy'];
        end
        if (exist('MembraneTimeCStore','var'))
            MembraneTimeCMoy(:,numPop)=mean(MembraneTimeCStore,2);
            MembraneTimeCStd(:,numPop)=std(MembraneTimeCStore,1,2);
            MembraneTimeCStdMoy(numPop)=mean(MembraneTimeCStd(:,numPop));
            MembraneTimeCPopMoy(numPop)=mean(MembraneTimeCMoy(:,numPop));
            nameSave=[nameSave, ' MembraneTimeCMoy MembraneTimeCPopMoy MembraneTimeCStd  MembraneTimeCStdMoy'];
        end
        if (exist('ThresholdStore','var'))
            ThresholdMoy(:,numPop)=mean(ThresholdStore,2);
            ThresholdStd(:,numPop)=std(ThresholdStore,1,2);
            ThresholdPopMoy(numPop)=mean(ThresholdMoy(:,numPop));
            ThresholdStdMoy(numPop)=mean(ThresholdStd(:,numPop));
            ThresholdPopMoy(numPop)=mean(ThresholdMoy(:,numPop));
            nameSave=[nameSave, ' ThresholdMoy ThresholdPopMoy ThresholdStd ThresholdStdMoy'];
        end
        %synapse
        if (exist('ExistenceStore','var'))
            ExistenceMoy(:,:,numPop)=mean(ExistenceStore,3);
            ExistenceStd(:,:,numPop)=std(ExistenceStore,1,3);
            ExistenceMoyNeuron(:,numPop)=mean(ExistenceMoy(:,:,numPop),2);
            ExistenceStdMoy(numPop)=mean(mean(ExistenceStd(:,:,numPop)));
            ExistencePopMoy(numPop)=mean(mean(ExistenceMoy(:,:,numPop)));
            nameSave=[nameSave, ' ExistenceMoy ExistencePopMoy ExistenceStd ExistenceStdMoy ExistenceMoyNeuron'];
        end
        if (exist('StrengthStore','var'))
            StrengthMoy(:,:,numPop)=mean(StrengthStore,3);
            StrengthStd(:,:,numPop)=std(StrengthStore,1,3);
             StrengthMoyNeuron(:,numPop)=mean(StrengthMoy(:,:,numPop),2);
            StrengthStdMoy(numPop)=mean(mean(StrengthStd(:,:,numPop)));
            StrengthPopMoy(numPop)=mean(mean(StrengthMoy(:,:,numPop)));
            nameSave=[nameSave, ' StrengthMoy StrengthPopMoy StrengthStd StrengthStdMoy  StrengthMoyNeuron'];
            
        end
        if (exist('DelayStore','var'))
            DelayMoy(:,:,numPop)=mean(DelayStore,3);
            DelayStd(:,:,numPop)=std(DelayStore,1,3);
             DelayMoyNeuron(:,numPop)=mean(DelayMoy(:,:,numPop),2);
            DelayStdMoy(numPop)=mean(mean(DelayStd(:,:,numPop)));
            DelayPopMoy(numPop)=mean(mean(DelayMoy(:,:,numPop)));
            nameSave=[nameSave, ' DelayMoy DelayPopMoy DelayStd DelayStdMoy  DelayMoyNeuron'];
        end
        if (exist('TimeCStore','var'))
            TimeCMoy(:,:,numPop)=mean(TimeCStore,3);
            TimeCStd(:,:,numPop)=std(TimeCStore,1,3);
              TimeCMoyNeuron(:,numPop)=mean(TimeCMoy(:,:,numPop),2);
            TimeCStdMoy(numPop)=mean(mean(TimeCStd(:,:,numPop)));
            TimeCPopMoy(numPop)=mean(mean(TimeCMoy(:,:,numPop)));
            nameSave=[nameSave, ' TimeCMoy TimeCPopMoy TimeCStd TimeCStdMoy  TimeCMoyNeuron'];
        end
        
        %mixed
        if (exist('ExistenceSignStore','var'))
            ExistenceSignMoy(:,:,numPop)=mean(ExistenceSignStore,3);
            ExistenceSignStd(:,:,numPop)=std(ExistenceSignStore,1,3);
             ExistenceSignMoyNeuron(:,numPop)=mean(ExistenceSignMoy(:,:,numPop),2);
            ExistenceSignStdMoy(numPop)=mean(mean(ExistenceSignStd(:,:,numPop)));
            ExistenceSignPopMoy(numPop)=mean(mean(ExistenceSignMoy(:,:,numPop)));
            nameSave=[nameSave, ' ExistenceSignMoy ExistenceSignPopMoy ExistenceSignStd ExistenceSignStdMoy ExistenceSignMoyNeuron'];
        end                                                                      
        
        
        %neuron
        clear  MembraneTimeCStore;
        clear SignStore;
        clear ThresholdStore;
        %synapse
        clear  ExistenceStore;
        clear  StrengthStore;
        clear DelayStore;
        clear TimeCStore;
        %mixed
        clear ExistenceSignStore;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        cd ..;
        display(['population number is ',num2str(numPop-1), ' with ', num2str(numIndi), ' individuals'])
        
    else
        display(['could not open directrory ', name])
    end
    indPop=indPop+1;
    if numPop >= maxNumPop
        break;
    end
end
name
cd ..;
eval(['save evolution ', nameSave, ' numPop']) %save variable


