% FUNCTION kern = calc_spike_kernel(delay, tau_s, tau_m, windowLen, updates, cycleTime)
%
%	Calculate the kernel in the same way than i
%
%		delay: delay before the spike is transmitted
%		tau_s: synaptic time constant
%		tau_m: post-synaptic neuron membrane time constant
%		windowLen: length of the kernel
%		updates: # update per cycle
%		cycleTime: cycle time in ms
%
%	Example:
%
%		plot(1:20, calc_spike_kernel(2, 10, 4, 20));
%
%		(default parameters used by i)

function [kern, t] = calc_spike_kernel(delay, tau_s, tau_m, windowLen, updates, cycleTime)

%kern(1:delay) = 0;
%kern(1:length(delay),(delay+1:windowLen)) = exp((-(delay+1:windowLen) + delay) ./ tau_m) .* (1 - exp((-(delay+1:windowLen)+ delay) ./ tau_s));


i = 1:windowLen;
t(i) = (i-1) .* cycleTime / updates;
kern = zeros(size(i));
kern = (t >= delay) .* exp(-(t - delay) / tau_m) .* (1 - exp(-(t - delay) / tau_s));
