% FUNCTION load_log(filename)
%
%	This function parse a log for i, and create coresponding matrix whose name is givent by the log legenda. The
%	log must satisfy this requirements:
%		- every thing right from a % character is ignored
%		- blank lines are ignored
%		- the first non comment, non blank line must be the legenda
%		- subsequant lines are treated as data
%		- if the legenda is repeated, it is ignored
%
%	The legenda must satify this requirements:
%		- each for each data column, a name must be present in legenda
%		- name have no white space
%		- if more than one column have the same name (ex: 25 column
%		representing pixels), the legenda name must indicate the number of
%		column it is representing with the syntax:
%
%				col_name#col_number (ex: pixels#25)
%
%	EXAMPLE: consider the following log file:
%
%		%log from today experiment
%		pix#4 result
%		2 4 5 2 6
%		3 5 2 1 6
%		%log stop
%		
%		%log start
%		pix#4 result
%		5 6 2 7 0
%
%	If the load_log function is called with this log file. The following
%	matrix will be created:
%
%	pix =
%     2     4     5     2
%     3     5     2     1
%     5     6     2     7
%
%	result =
%	  6
%	  6
%	  0

function load_log(filename)

fid = fopen(filename);
firstLine = 1;

colNum = -1;
lineNum = 0;
header = '';
cols = cellstr(['']);
colsNum(1) = 1;

while feof(fid) ~= 1
    line = fgetl(fid);
    
    % handle empty line, full comments and repeated header
    if(length(line) == 0 || line(1) == '%' || strcmp(header, line))
        continue;
    end
    
    % save header line
    if(firstLine == 1)
    	header = line;
    end
    
    % handle comments in line
    line = strtok(line, '%');
    
    if(firstLine == 1)
    	% here we must parse the header
        firstLine = 0;
        colNum = 0;
        
        % here we generate a string describing the creation of the cell
        % array of strings with the header label
        tmp = 'cols = {''';
        while length(line) > 0
            [token, line] = strtok(line);
  
            % handle some cases bad formed legenda.
            if(length(token) == 0)
            	continue;
            end
            
            colNum = colNum+1;
            
            % here we must check for a col length indicator
            [headername, token] = strtok(token, '#');
            
            % we save the col length indicator in colsNum
            if length(token) == 0
            	colsNum(colNum) = 1;
            else
            	token = strtok(token, '#');
            	colsNum(colNum) = str2num(token);
            end
            
            tmp = strcat(tmp, headername, ''',''');
        end
        tmp = strcat(tmp, '''};');
        
        % we evaluate the string to have the cell array created
        eval(tmp);
    else
        lineNum = lineNum + 1;
        
        % we are in a data line
        %colIndex = 0;
        for col = 1:colNum
        	for subCol = 1:colsNum(col)
        		%colIndex = colIndex + 1;
        		
            	[token, line]=strtok(line);
            	% here we want var(header(col))(lineNum) = toNum(token) 
            	
            	tmp2 = strcat(cols{col}, '(', int2str(lineNum), ',', int2str(subCol), ')=',token, ';');
            	evalin('caller', tmp2);
            end
        end
    end
end
fclose(fid);