%--------------------------------------------------------------------- 
%   Autonomous System Laboratory (LSA2), EPFL, Lausanne, Switzerland
%	Adaptive Intelligence Laboratory (AI Lab), University Utrecht 
%---------------------------------------------------------------------
%
%FUNTION plot_evolution(user)
%
%   load data from the file "evolution.mat" and plot them agianst generation
%   number. 
%
%   user=1 if present, i.e. if the function might ask the user some
%   paramters
%
%   It asks the user (if present): "What is the display time between each
%   generation [s]?" i.e. it is useful for synaptic parameters (like
%   synaptic existence or delay) because it allows to display every generation  
%   each after each other (4D graphs...)
%
%   You can improve the display or plot other parameters, that are
%   calculated with the function  ANALYSE_EVOLUTION
%
%   See also ANALYSE, ANALYSE_EVOLUTION
function plot_evolution(user)
if (nargin==0)
    user=1;
end
load evolution.mat
if (user==1)
    timeP = input('What is the display time between each generation [s]');
    if isempty(timeP)
        timeP=0.01;
    end
else
    timeP=0.01;
end

'***Sign*****'

if (exist('SignMoy','var'))
    figure;
    [s1,s2,s3]=size(SignMoy);
    for i=1:s1
        subplot(s1,1,i)
        bar(SignMoy(i,:,:))
        axis([1 s2 -1 1])

        if (i==1)
            title('Sign ')
        end
        Ylabel(['Neuron ', num2str(i)]); 
    end
    Xlabel('Generation Number [-]')
    %%%%%%
    %bar3(SignMoy)
    %title(['Sign: mean over population         generation number '])
    %Xlabel('generation number [-]');
    %Ylabel('neuron''s number [-]');
    %axis([1 s2 1 s1 -1 1])
    %%%%%%%5
    figure;
    plot(SignPopMoy,'b');
    hold on
    plot(SignStdMoy, 'r-.');
    title('Mean Sign (whole network) ')
    legend('Sign','Std');
    Xlabel('Generation Number [-]');
    axis([1 s2  -1 1])
end

'***Threshold***'

if (exist('ThresholdMoy','var'))
    figure;
    [s1,s2,s3]=size(ThresholdMoy);
    maxThreshold=max(max(ThresholdMoy));
    
    for i=1:s1
        subplot(s1,1,i)
        bar(ThresholdMoy(i,:,:))
        axis([1 s2 0 maxThreshold])
        Ylabel('Threshold[-]');  
        if (i==1)
            title('Threshold ')
        end
        Ylabel(['Neuron ', num2str(i)]); 
    end
    Xlabel('Generation Number [-]')
    %%%%%%%%%%%%%%%%%%%%%%
    %bar3(ThresholdMoy)
    %title(['Thresholds: mean over population         generation number '])
    %Xlabel('generation number [-]');
    %Ylabel('neuron''s number [-]');
    %axis([1 s2 1 s1  0 maxThreshold])
    %%%%%%%%%%%%%%%%%%%%%%%%%%
    figure;
    plot(ThresholdPopMoy);
    hold on
    plot(ThresholdStdMoy, 'r-.');
    title('Mean Trheshold (whole network)')
    legend('Threshold','Std');
    Xlabel('Generation Number [-]')
    axis([1 s2  0 maxThreshold])
end

if (exist('MembraneTimeCMoy','var'))
    figure;
    [s1,s2,s3]=size(MembraneTimeCMoy);
    maxMembraneTimeC=max(max(MembraneTimeCMoy));
    
    for i=1:s1
        subplot(s1,1,i)
        bar(MembraneTimeCMoy(i,:,:))
        axis([1 s2 0 maxMembraneTimeC])
        if (i==1)
            title('Membrane Time Constant ')
        end
        Ylabel(['Neuron ', num2str(i)]); 
    end
    Xlabel('Generation Number [-]')
    %%%%%%%%%%%%%%%%%%
    %bar3(MembraneTimeCMoy)
    %title(['MembraneTimeCMoy: mean over population         generation number '])
    %Xlabel('generation number [-]');
    %Ylabel('neuron''s number [-]');
    %axis([1 s2 1 s1  0 maxMembraneTimeC])
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    figure;
      plot(MembraneTimeCPopMoy);
      hold on
      plot(MembraneTimeCStdMoy, 'r-.');
      title('mean MembraneTimeC ')
      legend('MembraneTimeC','Std');
      axis([1 s2  0 maxMembraneTimeC])
end

'****Synapse exisence*****'

if (exist('ExistenceMoy','var'))
    [s1,s2,s3]=size(ExistenceMoy)
    figure;
    %%%%
    %  for i=1:numPop
    %     pause(0.1)
    %     bar3(ExistenceMoy(:,:,i))
    %    title(['Existence: mean value over the population            generation number ',num2str(i)])
    %   Xlabel('generation number [-]');
    %  Ylabel('neuron''s number [-]');
    % axis([1 s2 0 s1 0 1])
    %end     
    for i=1:numPop        
        pause(timeP)
        for j=1:s1
            subplot(s1,1,j)
            bar(ExistenceMoy(j,:,i))
            axis([1  s2 0 1])
            if (j==1)
                title(['Synapse Existence. Generation Number ',num2str(i)])
            end
            Ylabel(['Neuron ', num2str(j)]); 
        end
        Xlabel('Presynaptics neuron''s number [-]');
    end
    
    %%%% teh following graph style is only avaiable for Existence and could be copied to
    %%%% plot in the same way the synaptic Delay, TimeC, strength
    figure;
    for i=1:s1
        subplot(s1,1,i) 
        bar(ExistenceMoyNeuron(i,:,:))
        axis([1 s2 0 1])
        if (i==1)
            title('Mean Synaptic Existence (per post synaptic neuron)')
        end
        Ylabel(['Neuron ', num2str(i),' [-]']);
    end
    Xlabel('Generation''s number [-]')
    %%%%
    %%%%
    
    figure;
     plot(ExistencePopMoy);
     hold on 
     plot(ExistenceStdMoy, 'r-.');
     title('Mean Synaptic Existence (whole network)')
     legend('Existence','Std');
     Xlabel('Generation''s number [-]')
     axis([1 s2  0 1])
    
end

'*** synaptic delay***'

if (exist('DelayMoy','var'))
    figure;
    [s1,s2,s3]=size(DelayMoy);
    maxDelay=max(max(max(DelayMoy)));
    for i=1:numPop      
        pause(timeP)
        bar3(DelayMoy(:,:,i))
        title(['Delay: mean over population          generation number ', num2str(i)])
        Xlabel('Generation Gumber [-]');
        Ylabel('Neuron''s number [-]');
        axis([1 s2 0 s1 0 maxDelay])
    end   
    figure; 
    plot(DelayPopMoy);
    hold on
    plot(DelayStdMoy, 'r-.');
    title('mean Delay')
    legend('Delay','Std');
    axis([1 s3 0 maxDelay])
end

'*** synaptic Strenghth****'

if (exist('StrengthMoy','var'))
    figure;
    [s1,s2,s3]=size(StrengthMoy);
    for i=1:numPop    
        pause(timeP)
        bar3(StrengthMoy(:,:,i))
        title(['Strength: mean over population          generation number ', num2str(i)])
        Xlabel('generation number [-]');
        Ylabel('neuron''s number [-]')
        axis([1 s2 0 s1 0 1])
    end  
    figure; 
    plot(StrengthPopMoy);
    hold on
    plot(StrengthStdMoy, 'r-.');
    title('mean Strength')
    legend('Strength','Std');
    axis([1 s2  0 1])
end

'*** Synaptic Time constant***'

if (exist('TimeCMoy','var'))
    figure;
    [s1,s2,s3]=size(TimeCMoy)
    maxTimeC=max(max(max(TimeCMoy)));
    for i=1:numPop
        pause(timeP)
        bar3(TimeCMoy(:,:,i))
        title(['TimeC: mean over population          generation number ', num2str(i)])
        Xlabel('generation number [-]');
        Ylabel('neuron''s number [-]');
        axis([1 s2 0 s1 0 maxTimeC])
    end
    figure;
    plot(TimeCPopMoy);
    hold on  
    plot(TimeCStdMoy, 'r-.');
    title('mean TimeC')
    legend('TimeC','Std');
    axis([1 s3 0 maxTimeC])
end

'***Existence * Sign***'

if (exist('ExistenceSignMoy','var'))
    figure;
    [s1,s2,s3]=size(ExistenceSignMoy);
    for i=1:numPop
        
        pause(timeP)
        bar3(ExistenceSignMoy(:,:,i))
        title(['Mean Existence * Sign:        generation number ', num2str(i)])
        Xlabel('pre-synaptic neuron');
        Ylabel('pos-synaptic neuron ');
        axis([1 s2 0 s1 -1 1])
    end
    figure;
    for i=1:s1
        subplot(s1,1,i)
        bar(ExistenceSignMoyNeuron(i,:,:))
        axis([1 s3 -1 1])
      if (i==1)
            title('Mean Synaptic Existence x Neuron''s Sign')
        end
        Ylabel(['Neuron ', num2str(i)]); 
    end
    
    figure;   
    plot(ExistenceSignPopMoy);
    hold on
    plot(ExistenceSignStdMoy, 'r-.');
    title('mean Existence * Sign')
    legend('Existence *Sign','Std');
    Xlabel('generation number [-]');
    axis([1 s3  -1 1])
end


reply=input('Do you want to close all figures?','s')
if isempty(reply)
    reply ='Y' ;
end
if (strcmp(reply,'Y')  || strcmp(reply,'y'))
    close all
end


%%%%%%draft only forget about it%%%%%%%%%%%%%%%%%%
i_want=0;
if (i_want)
    if (exist('ExistenceMoy','var'))
        [s1,s2,s3]=size(ExistenceMoy);
        figure;
        for i=1:numPop      
            pause(0.1)
            bar3(ExistenceMoy(:,:,i))
            title(['Existence: mean value over the population            generation number ',num2str(i)])
            Xlabel('Generation number [-]');
            Ylabel('Neuron''s number [-]');
            axis([0 s1 0 s2 0 1])
        end
        figure;
        for i=1:numPop        
            pause(0.1)
            for j=1:4
                subplot(4,1,j)
                bar(ExistenceMoy(j,:,i))
                title(['Existence: mean value over the population            generation number ',num2str(i)])
                Xlabel('Generation number [-]');
                Ylabel('neuron''s number [-]');
            end
        end
        figure;
        for i=1:4
            for j=1:s3
                for k=1:s2
                    ExistenceMoyTemp(k,j)=ExistenceMoy(i,k,j);
                end
            end
            subplot(4,1,i)
            bar3(ExistenceMoyTemp)
            title(['Existence: mean value over the population          '])
            Xlabel('generation number [-]');
            Ylabel('neuron''s number [-]');
            clear ExistenceMoyTemp;
        end      
        figure(43)    
        for i=1:numPop
            pause(0.1)
            bar3(ExistenceStd(:,:,i))
            title(['Existence: Standard deviation          generation number ',num2str(i)])
        end
        figure;   
        plot(ExistencePopMoy);
        hold on      
        plot(ExistenceStdMoy, 'r-.');
        title('mean Existence')
        legend('Existence','Std'); 
    end
end

