/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "../status.h"
#include "neuron.h"

#include "neuralnetwork.h"

NeuralNetwork::NeuralNetwork(wxUint8 size, wxUint8 numOutput) {
	mSize = size;
	mNeurons = new Neuron *[mSize];
	for(wxUint8 i=0; i<mSize; i++)
		mNeurons[i] = NULL;
	
	mNumOutput = numOutput;
	mOutput = new float[mNumOutput];
	for(wxUint8 i=0; i<mNumOutput; i++)
		mOutput[i] = 0.0f;
		
	mCurrentCycle = 0;
}

NeuralNetwork::~NeuralNetwork() {
	delete [] mOutput;
	for(wxUint8 i=0; i<mSize; i++)
		if(mNeurons[i])
			delete mNeurons[i];
	delete [] mNeurons;
}

wxInt8 NeuralNetwork::SaveActivity(wxString &filename) {
	wxFile *file = new wxFile(filename.c_str(), wxFile::write);
	if(!file->IsOpened()) {
		StatusLine("NeuralNetwork - Couldn't open file \"" + filename + "\" for writing.");
		delete file;
		return 1;
	}
	SaveActivity(file);
	file->Close();
	delete file;
	return 0;
}

wxSize NeuralNetwork::GetPaintSize() {
	wxSize architectureSize , activitySize , sumSize;
	architectureSize = GetPaintArchitectureSize();
	activitySize = GetPaintActivitySize();
  
	sumSize.SetWidth( architectureSize.GetWidth() + activitySize.GetWidth() );
	sumSize.SetHeight( 
		// the max
		activitySize.GetHeight() > architectureSize.GetHeight() ?
		activitySize.GetHeight() : architectureSize.GetHeight() 
	);
	
	return sumSize;
}

wxSize NeuralNetwork::GetPaintArchitectureSize() {
	return wxSize(0,0) ;
}

wxSize NeuralNetwork::GetPaintActivitySize() {
	return wxSize(0,0) ;
}

void NeuralNetwork::Paint(wxDC& dc, wxPoint& position, wxSize& size) {
	wxUint16 boundary = (wxUint16)(0.6f * size.GetWidth());

	/* Draw neural network architecture. */
	position.x += boundary;
	size.SetWidth(size.GetWidth() - boundary);
	dc.SetClippingRegion(position.x, position.y, size.GetWidth(), size.GetHeight());
	PaintArchitecture(dc, position, size);
	dc.DestroyClippingRegion();

	/* Draw neural network activity. */
	position.x -= boundary;
	size.SetWidth(boundary);
	dc.SetClippingRegion(position.x, position.y, size.GetWidth(), size.GetHeight());
	PaintActivity(dc, position, size);
	dc.DestroyClippingRegion();	
}
