/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>
#include <math.h>

#include "perceptron.h"

Perceptron::Perceptron(wxUint8 inputSize, wxUint8 outputSize) : NeuralNetwork(0, outputSize) {
	mInputSize = inputSize;
	mWeights = NULL;
}

Perceptron::~Perceptron() {
	if(mWeights)
		delete [] mWeights;
}

void Perceptron::Compute() {
	wxUint8 i, j;
	for(i=0; i<mNumOutput; i++) {
		mOutput[i] = 0.0f;
		for(j=0; j<mInputSize; j++)
			mOutput[i] += mInput[j] * mWeights[i * mInputSize + j];
		mOutput[i] += mWeights[mNumOutput * mInputSize + i]; // bias
		mOutput[i] = 1.0f / (1.0f + exp(-mOutput[i]));
	}
}

void Perceptron::PaintArchitecture(wxDC& dc, wxPoint& position, wxSize& size) {
	/* Draw neural network name */
	wxFont font = dc.GetFont();
	font.SetWeight(wxBOLD);
	dc.SetFont(font);
	dc.DrawText("Perceptron", position.x + 10, position.y + 15);
	font.SetWeight(wxNORMAL);
	dc.SetFont(font);

	position.x += 10;
	position.y += 50;
	size.SetHeight(size.GetHeight() - 35);
	size.SetWidth(size.GetWidth() - 20);
}
