/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _PERCEPTRON_H
#define _PERCEPTRON_H

#include "../neuralnetwork.h"

/** 
 * Simple two-layered, feed-forward sigmoidal neural network.
 */
class Perceptron : public NeuralNetwork
{
public:
	/**
	 * Default constructor.
	 * @param inputSize the number of input neurons
	 * @param outputSize the number of output neurons
	 */
	Perceptron(wxUint8 inputSize, wxUint8 outputSize);

	/** Destructor. */
	~Perceptron();  

	/**
	 * Method to set the input values into the input neurons of the neural network
	 * @param input is an array of inputs 
	 */
	virtual void SetInput(float *input) { mInput = input; }

	void SetWeights(float *weights) { mWeights = weights; }

	/**
	 * Method to compute the output values of the neural network 
	 */
	virtual void Compute();
  
	/** set storage of activity*/
	virtual void AllocActivityStorage(wxUint16 size) {}

	/* ---------------- LOGGING -------------------- */

protected:
	/** Does the actual saving of the neural activity. */
	virtual void SaveActivity(wxFile *file) {}

public:
	/* ---------------- PAINTING -------------------- */

	/** 
	 * Paint the neural network architecture, which is neural network dependent. 
	 * @param dc Device Context to use.
	 * @param position Upperleft corner of drawing area.
	 * @param size Size of drawing area.
	 */
	virtual void PaintArchitecture(wxDC& dc, wxPoint& position, wxSize& size);

	/** 
	 * Paint the neural network activity, which is neural network dependent.
	 * @param dc Device Context to use.
	 * @param position Upperleft corner of drawing area.
	 * @param size Size of drawing area.
	 */
	virtual void PaintActivity(wxDC& dc, wxPoint& position, wxSize& size) {}

protected:
	wxUint8 mInputSize;
	float *mInput;
	float *mWeights;
};

#endif /* _PERCEPTRON_H */
