/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _PERCEPTRONFACTORY_H
#define _PERCEPTRONFACTORY_H

class Genome;
class NeuralNetwork;
#include "../neuralnetworkfactory.h"

/**
 * Concrete NN factory, able to create SimpleNNs.
 * @see Genome
 * @see Experiment
 */
class PerceptronFactory : public NeuralNetworkFactory
{
public:
	PerceptronFactory(ModelSettings *modelSettings);
	virtual ~PerceptronFactory();

	/**
	 * Method to create a neural network from a genome and an experiment
	 * @param genome genome encoding parameters evolved
	 * @return a neural network of the wanted type
	 */
	virtual NeuralNetwork *Create(Genome *genome);

	/** Returns the genome length required with current genome encoding. */
	virtual wxUint16 GetGenomeLength();
	
	/** 
	 * Configure the genome encoding settings. 
	 * @return The new required genome length (zero on error).
	 */
	virtual wxUint16 ConfigGenomeEncoding();

	/** Configure neural network specific settings. */
	virtual void ConfigNeuralNetwork();

	/** Returns the type of neural network the factory can create. */
	virtual NeuralNetworkType GetType() { return NN_PERCEPTRON; }
	
	/* XML stuff */
	virtual void			Load (XMLElement *myself, XMLLoader *loader);
	virtual XMLElement*		SaveSelf ();
	virtual wxString		GetXMLName ();
	virtual wxString		GetShortXMLName();
	static wxString			GetStaticXMLType() { return "nn-perceptron"; }
	static const wxString	shortXMLName;

protected:
	wxUint8 mWeightNumBits; /**< Number of bits encoding one weight. */
};

#endif /* _PERCEPTRONFACTORY_H */
