/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _SPIKINGNNFACTORY_H
#define _SPIKINGNNFACTORY_H



#include "../../utils/xml/xml.h"
#include "../neuralnetworkfactory.h"
#include "../neuralnetworktopology.h"



class Genome;
class NeuralNetwork;
class SpikingReceptor;
class SpikingOutputAC;
class SpikingNNTopology;
class GenomeLayout;
class Neuron;
class Synapse;

/*enum ReceptorType {
	SPIKING_RECEPTOR_NEW_NOREFRAC = 1,		// determine spike stochastically every ms, without refractory period
	SPIKING_RECEPTOR_NEW_REFRAC,			// determine spike stochastically every ms, with refractory period
	SPIKING_RECEPTOR_DELAY					// Delay coding. 
};

enum OutputACType {
	SPIKING_OUTPUT_AC_RATE = 1,		// Rate coding
	SPIKING_OUTPUT_AC_DELAY,			// Delay coding
};*/

enum SpikingNNTopologyType
{
	SPTOPO_FULLY_CONNECTED = 1,
	SPTOPO_FULLY_CONNECTED_NO_RECURENT_OUTPUT,	///< Only internal conne
	SPTOPO_FEEDFORWARD_RECURENT,
	SPTOPO_FEEDFORWARD,
	SPTOPO_NO_DIRECT_CONNECT_NO_RECURENT,
	SPTOPO_NO_DIRECT_CONNECT,
};

/**
 * Concrete NN factory, able to create spiking NeuralNetworks. 
 * @see Genome
 * @see Experiment
 * @see NeuralNetwork
 */
class SpikingNNFactory : public NeuralNetworkFactory
{
public:
	SpikingNNFactory(ModelSettings *modelSettings);
	virtual ~SpikingNNFactory();

	//virtual void ReadGenome(Genome *genome);

	/**
	 * Method to create a neural network from a genome and an experiment
	 * @param genome genome encoding parameters evolved
	 * @return a neural network of the wanted type
	 */
	virtual NeuralNetwork *Create(Genome *genome);


	/** Returns the genome length required with current genome encoding. */
	virtual wxUint16 GetGenomeLength();
	
	/** 
	 * Configure the genome encoding settings. 
	 * @return The new required genome length (zero on error).
	 */
	virtual wxUint16 ConfigGenomeEncoding();

	/** Configure neural network specific settings. */
	virtual void ConfigNeuralNetwork();

	/** Log NN evolved paramameters (i.e. the matrix mSign, mExistence,..) 
	*	for every generaion, and every individual into file of type "genome_M_N.log",
	*	with M is the population number and N the individual number
	*	The saved file can be read using the matlbab script load_log.m, available in cvs ../matlab.
	**/
	virtual void LogNN(Genome *genome, NeuralNetwork* nn);

	
	/** Returns the type of neural network the factory can create. */
	virtual NeuralNetworkType GetType() { return NN_SPIKING; }
	

	/* XML stuff */
	virtual void			Load (XMLElement *myself, XMLLoader *loader);
	virtual XMLElement*		SaveSelf ();	
	virtual wxString		GetXMLName ();
	virtual wxString		GetShortXMLName();
	static wxString			GetStaticXMLType() { return "nn-spiking"; }
	static const wxString	shortXMLName;

protected:
	
	// THESE SHOULD BE OVERRIDES FROM NNFACT
	
	
	/** Return the topology to be used. */
	virtual NeuralNetworkTopology* GetNNTopology();
	
	/** Return the genome layout object. */
	virtual GenomeLayout* GetGenomeLayout();
	
	/** Create the neural network. */
	virtual NeuralNetwork* CreateNeuralNetwork();
	
	/**
	 * Create a neuron.
	 * @param type Type of the neuron to be created.
	 */
	virtual Neuron* CreateNeuron(NeuronType type);
	
	/**
	 * Create a synapse.
	 * @param preNeuron Pre-synaptic neuron.
	 * @param postNeuron Post-synaptic neuron.
	 * @param type Type of the synapse to be created.
	 */
	virtual Synapse* CreateSynapse(Neuron* preNeuron, Neuron* postNeuron, SynapseType type);	




	NeuronType mReceptorType; /**< Spiking receptor to use. */
	NeuronType mOutputACType; /** spiking ouput neuron type to use*/
	
	wxUint8 mInternalSize;		///< Number of internal neuron (without output neurons).
	

	SpikingNNTopologyType mTopologyType;	///< Constant to identify the topology type.
	
	// Helper object.
	GenomeLayout* mGenomeLayout;		///< Genome layout.
	SpikingNNTopology* mNNTopology;		///< Neural network topology object.
	bool mParameterChanged;				///< Flag indicating if parameter have changed.

};

#endif /* _SPIKINGNNFACTORY_H */
