/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/



#ifndef _SPIKINGNNTOPOLOGY_H
#define _SPIKINGNNTOPOLOGY_H

#include "../neuralnetworktopology.h"
#include "spikingnnfactory.h"





class SpikingNNTopology : public NeuralNetworkTopology
{
public:

	SpikingNNTopology(wxUint32 numInternal, wxUint32 numOuput, wxUint32 numSensor,
			NeuronType inputCoding, NeuronType outputCoding, SpikingNNTopologyType topoType);
	
	virtual ~SpikingNNTopology() {}
	
	
	
	// Overrides.
	virtual wxUint32 GetOutputNumber() {return mNumAC;}
	virtual wxUint32 GetOutputNeuronIndex(wxUint32 outputNum) {return mNumInternal + mNumOutput + outputNum;}
	
	
	virtual wxUint32 GetInternalNumber() {return mNumInternal;}
	virtual wxUint32 GetInternalNeuronIndex(wxUint32 internalNum) {return mNumInternal;}
	
			// helper functions.
/*	virtual void SetInternalNeuronInfo(wxUint32 neuron, NeuronType type, EvolvabilityType prop, wxUint16 neuronClass) {SetNeuronInfo(neuron, type, prop, neuronClass);}
	virtual void SetOutputNeuronInfo(wxUint32 neuron, NeuronType type, EvolvabilityType prop, wxUint16 neuronClass) {SetNeuronInfo(mNumInternal+neuron, type, prop, neuronClass);}
	virtual void SetACNeuronInfo(wxUint32 neuron, NeuronType type, EvolvabilityType prop, wxUint16 neuronClass) {SetNeuronInfo(mNumInternal+mNumOutput+neuron, type, prop, neuronClass);}
*/	
	// GOES IN NNTOPO!!!
//	virtual void SetSensorNeuronInfo(wxUint32 neuron, NeuronType type, EvolvabilityType prop, wxUint16 neuronClass) {SetNeuronInfo(mNumNeurons+neuron, type, prop, neuronClass);}
	
	
	
	//virtual void SetOutputNeuronConnectivity(EvolvabilityType
	
protected:

	virtual void SetupNeurons();
	virtual void SetupSynapses();

	NeuronType mInputCoding;
	NeuronType mOutputCoding;
	
	SpikingNNTopologyType mTopologyType;
	
	wxUint32 mNumInternal;
	wxUint32 mNumOutput;
	wxUint32 mNumAC;
	
};


#endif
