/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "spikingoutputac.h"
#include "spikingsynapse.h"
#include "spikingneuron.h"


SpikingOutputAC::SpikingOutputAC(){
	
	mOutputAlloc = NULL;

}
SpikingOutputAC::~SpikingOutputAC(){
	
	if(mOutputAlloc)
		delete[] mOutputAlloc;
}

void SpikingOutputAC::AllocActivityStorage(wxUint16 numCycles, wxUint8 numUpdates)
{
	SpikingBase::AllocActivityStorage(numCycles, numUpdates);
	mOutputAlloc = new float[mMaxNumCycles];
	
}

float SpikingOutputAC::ComputeExternalPotential(wxUint16 currentUpdate){
	/*The Output Neuron does not use external potential*/
	return 0;
}


void SpikingOutputAC::ComputeMembranePotential(wxUint16 currentCycle, wxUint16 currentUpdate){

	float potential = 0.0f;

	// compute presynaptic composant
	size_t count = mIncoming->GetCount();
	Synapse *synapse;
	Neuron *preNeuron;
	for(wxUint16 i=0; i<count; i++) {
		synapse = mIncoming->Item(i);
		preNeuron = synapse->GetPreNeuron();
		potential+=((SpikingNeuron *)preNeuron)->GetActivity(currentCycle, currentUpdate);
	}
	
	mActivity[currentCycle][currentUpdate] = potential > 0.0f;
	wxASSERT_MSG(potential==0 || potential==1,"failure with the output neuron membrane potential computation");
}
void SpikingOutputAC::PaintActivity(wxDC& dc, wxPoint& position, wxUint16 cyclesVisible, wxUint16 currentCycle, wxUint8 updatesPerCycle) {
	//To be improved...
	wxUint16 x, start = 0;
	wxUint8 update;
	if(currentCycle > cyclesVisible)
		start = currentCycle - cyclesVisible;
	x = position.x + 20;
		if(currentCycle > cyclesVisible)
		start = currentCycle - cyclesVisible;
	for(wxUint16 cycle = start; cycle<currentCycle; cycle++){
		for(update = 0; update < (updatesPerCycle/NumUpdatesToDrawRatio); update++) {
			//dc.DrawLine(x, position.y + 18 - mOutputAlloc[cycle] * SpikeDrawHeight , x+(DisplaySpaceSpikes), position.y + 18);
			dc.DrawLine(x, (int) (position.y + 18 - mOutputAlloc[cycle] * SpikeDrawHeight), x+(DisplaySpaceSpikes), position.y + 18);
			x += DisplaySpaceSpikes;
		}
		dc.DrawLine(x, (int) (position.y + 18 - mOutputAlloc[cycle] * SpikeDrawHeight), x+(DisplaySpaceSpikes), position.y + 18);
		x += DisplaySpaceSpikes;
	}
	position.y += SpikeDrawHeight + 5;

}
