/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _SPIKINGOUTPUTAC_H
#define _SPIKINGOUTPUTAC_H

#include "spikingbase.h"

/* The SpikingOutputAC is a copy of the old named "output neurons". The difference is that,
 the SpikingOutputAC have got no dynamic at all, It does only copy the activity 
 (with the help of the spikingOutputSynapse), and convert it to an analogue value 
 readable by the motors of the robot. The SpikingOutputAC is in some way the symmetrical
 of the (input) receptors. Note that receptor convert an analogue value to digital datas (spikes). 
 M.S.
 */


/** 
 * Abstract class for a Spiking Output Analogue Converter.
 */
class SpikingOutputAC : public SpikingBase
{
public:
	/*constructor*/
	SpikingOutputAC();
	/*destructor*/
	virtual ~SpikingOutputAC();

	/*Get presynaptic components, through the spikingouputsynapse, and updates neruon's activity*/
	virtual void ComputeMembranePotential(wxUint16 currentCycle, wxUint16 currentUpdate);

	/*Output does not have external contribution, but the activity of the corresponding internal neuron is
	directly transmitted to the neuron through the spikingouputsynapse*/
	virtual float ComputeExternalPotential(wxUint16 currentCycle); 
	
	/*Compute an analogue ouptut value */
	virtual void ComputeOutput(wxUint16 currentCycle,wxUint16 updatesPerCycle,wxUint16 outputWindowLen)=0;

	/*Paint the activity converted through the Analogue Converter*/
	virtual void PaintActivity(wxDC& dc, wxPoint& position, wxUint16 cyclesVisible, wxUint16 currentCycle, wxUint8  updatesPerCycle);

	/*Accessor*/
	virtual float GetOutput() {return mOutput;}
	
	/** 
	 * Allocates storage for the activity of a certain number of network updates. 
	 * @param numCycles Number of cycles to reserve space for.
	 * @param numUpdates Number of network updates per cycle to reserve space for.
	 */
	virtual void AllocActivityStorage(wxUint16 numCycles, wxUint8 numUpdates);
	
protected:
	float mOutput; /*Analogue neuron's output*/
	float *mOutputAlloc;
	
};

#endif /* _SPIKINGOUTPUTAC_H */
