/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _SPIKINGRECEPTORNEWNOREFRAC_H
#define _SPIKINGRECEPTORNEWNOREFRAC_H

#include "../../utils/randomtools.h"

#include "spikingreceptor.h"

/** 
 * Spiking receptor new style, no refractory period.
 */
class SpikingReceptorNewNoRefrac : public SpikingReceptor
{
public:

	virtual ~SpikingReceptorNewNoRefrac() {}
	
protected:
	/** Computes external contributions to the membrane potential. */
	virtual float ComputeExternalPotential(wxUint16 currentUpdate) { return 1.0f * RandomTools::Flip(mExternalCurrent); }

	/** Computes the refractory kernel to add to the membrane potential. */
	virtual float ComputeRefractoryKernel() { return 0.0f; }
};

#endif /* _SPIKINGRECEPTORNEWNOREFRAC_H */
