/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _SPIKINGSYNAPSE_H
#define _SPIKINGSYNAPSE_H

#include <wx/wx.h>
#include "../synapse.h"



enum
{
	PARAMID_SPIKE_DELAY = 1100,
	PARAMID_SPIKE_SYNAPSE_TIME_cONST
};

/** 
 * The class to construct an artificial spiking synapse, inherited from Synapse
 * @see SpikingNeuron
 */
class SpikingSynapse : public Synapse
{
public:
	/**
	 * SpikingSynapse constructor
	 * @param presynactic departure neuron of this synapse  
	 * @param postsynaptic arrival neuron of this synapse
	 */
	SpikingSynapse(Neuron *preNeuron, Neuron *postNeuron);

	/** SpikingSynapse destructor */ 
	virtual ~SpikingSynapse();  
	

	/* --------------------- ACCESSORS ------------------------- */

	/** @param delay the delay of transmission along the synapse */
	void SetDelay(wxUint8 delay) { mDelay = delay; mMustBuildKernel = true;}
	/** @return the delay of transmission along the synapse */
	wxUint8 GetDelay() { return mDelay; }

	/** @param timeC time constant of the synapse */
	void SetTimeC(float timeC) { mTimeC = timeC; mMustBuildKernel = true;}
	/** @return the time constant of the synapse */
	float GetTimeC() { return mTimeC; }

	/* ------------------ COMPUTATION ---------------------------- */

	/** Init the refractory kernel values of this synapse */
	void InitKernel();

	/** Compute this synapse's contribution to the membrane potential of the postsynaptic neuron. */
	float ComputePotential(wxUint16 spikeWindowIdx);
	
	/** Perform learning. */
	virtual void PerformLearning() {}
	
	virtual void PaintArchitecture(wxDC &dc, wxPoint& position, wxSize& size);


protected:
	wxUint8 mDelay; /**< the delay of transmission along the synapse */
	float mTimeC; /**< the time constant of the synapse */
	float *mKernel; /**< Precache of refractory kernel values. */
	bool mMustBuildKernel; /**< True if the kernel must be recomputed. */
	
	
	virtual void SetParameterSelf(ParamID id, float param);
};

#endif /* _SPIKINGSYNAPSE_H */
