/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _SYNAPSE_H
#define _SYNAPSE_H

#include "../utils/iparameterizable.h"

class Neuron;

enum
{
	PARAMID_STRENGTH = 1000,
	PARAMID_EXISTENCE
};

/** 
 * Base synapse.
 * @see Neuron
 * @see NeuralNetwork
 */
class Synapse : public IParameterizable
{
public:
	/**
	 * Synapse constructor
	 * @param presynactic departure neuron of this synapse  
	 * @param postsynaptic arrival neuron of this synapse
	 * @param strength weigth of the connection
	 */
	Synapse(Neuron *preNeuron, Neuron *postNeuron, float strength);

	/** 
	 * Synapse destructor
	 */ 
	virtual ~Synapse(); 
	

	/** 
	 * Accessor method
	 * @return strength value
	 */
	float GetStrength() { return (mExists ? mStrength : 0.0f); }

	/** 
	 * Accessor method
	 * @param strength value of strength
	 */
	void SetStrength(float strength) { mStrength = strength; }
	
	/**
	 * Accessor method
	 * @return is the synapse connected
	 */
	bool GetExistence() {return mExists;}

	/**
	 * Accessor method
	 * @param existence is the synapse connected
	 */
	void SetExistence(bool existence) {mExists = existence;}

	/** 
	 * @brief Accessor method
	 * @return departure neuron
	 */
	Neuron* GetPreNeuron() { return mPreNeuron; }
  
	/** 
	* Accessor method
	* @return arrival neuron
	*/
	Neuron* GetPostNeuron() { return mPostNeuron; }
	
	virtual void PaintArchitecture(wxDC &dc, wxPoint& position, wxSize& size) {}
  
protected:
	float mStrength; /**< strength of this synapse */
	bool mExists; /**< is this synapse connected? */
	Neuron* mPreNeuron; /**< departure neuron of this synapse */
	Neuron* mPostNeuron; /**< arrival neuron of this synapse */
	
	
	virtual void SetParameterSelf(ParamID id, float param);
};

#endif /* _SYNAPSE_H */
