/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include "register.h"

// This is were CommLinks, Robots and Experiments should be registered
// to be made available. The magic of this mechanisme is contained in
// the RegisterXXX macros and in the FactoryEngine template classes. The
// various factories (CommLinkFactory, RobotFactory and ExperimentFactory)
// are very simple interface to the engine.

// Thanks to this mechanism, the only dependence between the core of the
// program (Controller, etc) and a particular module (SerialPort, eg) is
// the inclusion and the register line in this file. If these lines are
// removed, the linker will not include the particular module in the
// executable, and thus the corresponding files can be removed from the 
// project. (This is not the case with SerialPort, eg, because it is
// explicitly used by Bluetooth, for example, but you get the idea...)



void Register::RegisterAll()
{
	RegisterCommLinks();
	RegisterRobots();
	RegisterExperiments();
}


/* ------------------------------ COMMLINKS ----------------------------- */

#include "comm/commlinkfactory.h"

#include "comm/serialport.h"
#include "comm/bluetooth.h"
#include "comm/webots.h"

void Register::RegisterCommLinks()
{
	RegisterCommLink(SerialPort);
	RegisterCommLink(Bluetooth);
	RegisterCommLink(Webots);
}



/* ------------------------------ ROBOTS ----------------------------- */

#include "robot/robotfactory.h"

#include "robot/khepera.h"
#include "robot/koala.h"
#include "robot/dummy.h"

void Register::RegisterRobots()
{
	RegisterRobot(Khepera);
	RegisterRobot(Koala);
	RegisterRobot(Dummy);
}



/* ------------------------------ EXPERIMENTS ----------------------------- */

#include "exp/experimentfactory.h"

#include "exp/kheperaexperiment.h"
#include "exp/kheperavisionexperiment.h"
#include "exp/timeseriesexperiment.h"
#include "exp/dummyexperiment.h"
#include "exp/koalaexperiment.h"

void Register::RegisterExperiments()
{
	RegisterExperiment(KheperaExperiment, Khepera);
	RegisterExperiment(KheperaVisionExperiment, Khepera);
	RegisterExperiment(TimeSeriesExperiment, Dummy);
	RegisterExperiment(DummyExperiment, Dummy);
	RegisterExperiment(KoalaExperiment, Koala);
}
