/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "../status.h"
#include "../email.h"
#include "../utils/iutils.h"
#include "../utils/xml/xml.h"
#include "robot.h"


#include "dummy.h"

const wxString Dummy::ItemDesc = "Dummy [no communication]";

Dummy::Dummy() {
}

Dummy::~Dummy() {

}

wxInt8 Dummy::Connect() {
	return 0;
}

void Dummy::Initialize() {

}

wxInt8 Dummy::ReadSensors() {
	SetChanged();
	return 0;
}

wxInt8 Dummy::SetMotors(float *speeds) {
	for(int i=0; i<3; i++) {
		if(speeds[i] < -1.0f) speeds[i] = -1.0f;
		if(speeds[i] >  1.0f) speeds[i] =  1.0f;
	}
	return 0;
}

wxInt8 Dummy::SetLED(wxUint8 num, bool enable) {
	// Empty implementation
	return 0;
}

wxInt8 Dummy::ManualControl(int *jsButtons, float *jsAxes) {
	wxInt8 ret = 0;
	return ret;
}

wxInt8 Dummy::ExecCommand(wxUint8 cmd) {
	wxInt8 ret = 0;
	return ret;
}

void Dummy::LogLegenda(Logger *logger) {
}

void Dummy::Log(Logger *logger) {
}

void Dummy::Configure() {

}

void Dummy::PaintSensorData(wxDC& dc, wxPoint& position, wxSize& size) {
	dc.SetPen(*wxBLACK_PEN);
}


/* --- XML --- */
void Dummy::Load (XMLElement *myself, XMLLoader *loader) {
	Robot::Load(myself,loader);
}

XMLElement* Dummy::SaveSelf () {	
	XMLElement *result = Robot::SaveSelf();
	result->SetName(GetShortXMLName());
	return result;
}

wxString Dummy::GetXMLName () { return GetShortXMLName(); }
wxString Dummy::GetShortXMLName() { return Dummy::GetStaticXMLType(); }
