/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "../../status.h"
#include "../../utils/xml/xml.h"
#include "../khepera.h"

#include "kheperaimpl.h"
#include "basickhep.h"
#include "evopickhep.h"

const wxUint8 KheperaImpl::mNumKhepTypes = 3;

const wxString KheperaImpl::mXMLKheperaNames[mNumKhepTypes] = {
	KheperaImpl::GetStaticXMLType(),
	BasicKhep::GetStaticXMLType(),
	EvopicKhep::GetStaticXMLType()
};

KheperaImpl::KheperaImpl(wxUint32 *sensorMask) {
	mType = KHEPERA_UNDEFINED;
	mpSensorMask = sensorMask;
}

KheperaImpl::~KheperaImpl() {

}

KheperaType KheperaImpl::GetKheperaTypeByXMLTagName(wxString tagName) {
	for(wxInt32 i=0; i< mNumKhepTypes; i++)
		if (tagName.IsSameAs(mXMLKheperaNames[i])) {return (KheperaType) (i);}
	return (KheperaType) 0;
}


/* --- XML --- */
void KheperaImpl::Load (XMLElement *myself, XMLLoader *loader) {
}

XMLElement* KheperaImpl::SaveSelf () {
	XMLElement *result = new XMLElement(GetShortXMLName());
	return result;
}

void KheperaImpl::SaveChilds (XMLSaver *saver) {
}

void KheperaImpl::SetParentName (const wxString &parentName) {
}

wxString KheperaImpl::GetXMLName () { return GetShortXMLName(); }
wxString KheperaImpl::GetShortXMLName() { return KheperaImpl::GetStaticXMLType(); }
