/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "robotfactory.h"
#include "../comm/commlink.h"



FactoryEngine<Robot, int>  RobotFactory::mEngine;



wxUint32 RobotFactory::GetIndexByID(int id)
{
	wxUint32 index = 0;
	mEngine.GetIndexByID(id, index);
	return index;
}


Robot* RobotFactory::Create(int id, CommLink* commLink)
{
	Robot* robot = mEngine.CreateItemWithID(id);
	if(robot != NULL)
	{
		robot->SetID(id);
		robot->SetCommLink(commLink);
	}
	return robot;
}

Robot* RobotFactory::CreateByIndex(wxUint32 index, CommLink* commLink)
{
	Robot* robot = mEngine.CreateItemWithIndex(index);
	if(robot != NULL)
	{
		robot->SetID(GetIDByIndex(index));
		robot->SetCommLink(commLink);
	}
	return robot;
}

Robot* RobotFactory::Create(const char* xmlName, CommLink* commLink)
{
	int id;
	Robot* robot = NULL;
	if(mEngine.GetIDByXMLName(xmlName, id))
		robot = mEngine.CreateItemWithID(id);
	if(robot != NULL)
	{
		robot->SetID(id);
		robot->SetCommLink(commLink);
	}
	return robot;
}





/*

#include "robot.h"
#include "khepera.h"
#include "koala.h"
#include "blimp2.h"
#include "slowflyer.h"
#include "pioneer1.h"
#include "lemon.h"
#include "dummy.h"



const wxUint8 RobotFactory::mNumTypes = 7;

const RobotType RobotFactory::mTypes[mNumTypes] = {
	ROBOT_KHEPERA,
	ROBOT_KOALA,
	ROBOT_BLIMP2,
	ROBOT_SLOWFLYER,
	ROBOT_PIONEER1,
	ROBOT_LEMON,
	ROBOT_DUMMY
};

const wxString RobotFactory::mDescs[mNumTypes] = {
	"Khepera",
	"Koala",
	"Blimp II",
	"SlowFlyer",
	"Pioneer 1",
	"Lemon",
	"Dummy"
};

const wxString RobotFactory::mXMLNames[mNumTypes] = {
	Khepera::GetStaticXMLType(),
	Koala::GetStaticXMLType(),
	Blimp2::GetStaticXMLType(),
	SlowFlyer::GetStaticXMLType(),
	Pioneer1::GetStaticXMLType(),
	Lemon::GetStaticXMLType(),
	Dummy::GetStaticXMLType()
};

RobotType RobotFactory::GetTypeByXMLTagName(wxString tagName) {
	for(wxInt32 i=0; i<mNumTypes; i++)
		if (tagName.StartsWith(mXMLNames[i])) {return mTypes[i];}
	return ROBOT_UNDEFINED;
}

Robot* RobotFactory::Create(RobotType type, CommLink *commLink) {
	switch (type) {
		case ROBOT_KHEPERA: 	return new Khepera(commLink); 	break;
		case ROBOT_KOALA: 		return new Koala(commLink); 	break;
		case ROBOT_BLIMP2:		return new Blimp2(commLink);  	break;
		case ROBOT_SLOWFLYER:	return new SlowFlyer(commLink); break;
		case ROBOT_PIONEER1: 	return new Pioneer1(commLink); 	break;
		case ROBOT_LEMON: 		return new Lemon(commLink); 	break;
		case ROBOT_DUMMY: 		return new Dummy(commLink); 	break;
		default:		break;
	}
	return NULL;
}
*/