/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _IPARAMETERIZABLE_H
#define _IPARAMETERIZABLE_H

#include <wx/wx.h>

#include <vector>
#include <map>
#ifdef WIN32
	#include <wx/msw/winundef.h>
#endif

typedef int ParamID;

/** Parameter value record. */
struct ParamRecord
{
	ParamID id;		///< Parameter ID.
	float value;	///< Used to encode floating point value as well as digital value.
};

class IParameterizable
{
public:

	IParameterizable() {}
	virtual ~IParameterizable() {}

		
	virtual void SetParameter(ParamID id, float param);
	
	virtual void SetParameters(const std::vector<ParamRecord> &params)
	{
		for(size_t i = 0; i < params.size(); i++)
			SetParameter(params[i].id, params[i].value);
	}
	
	virtual float GetParameter(ParamID id) {return mParameters[id];}

protected:

	std::map<ParamID, float> mParameters;

	virtual void SetParameterSelf(ParamID id, float param) {}
	
};

#endif

