/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _IUTILS_H
#define _IUTILS_H

/* ------------------------ SLEEP --------------------------- */

/* Always use this sleep function to make sure that the correct
 * sleep function is called. This depends on the current thread.
 */

/** Sleep for the given number of milliseconds. Calls either
 *  wxThread::Sleep() or wxUsleep(), depending on current thread.
 */
void iSleep(unsigned long milliseconds);

/* ------------------------ YIELD --------------------------- */

/** Use this function if you want to give the GUI the opportunity
 *  to update.
 */
void iYield();

/* ------------------------ MESSAGEBOX --------------------------- */

/** Thread-safe messagebox. */
int iMessageBox(const wxString& message, const wxString& caption, int style);

/**
 * Ask the user for a float number
 * @param message Message displayed.
 * @param caption Caption displayed.
 * @param value On input, the default value proposed. On output, the value choosen by the user.
 * @return Return true if the user acknowledged AND the text conversion could be done. If false, value is not modified.
 */
bool iGetFloatFromUser(const wxString &message, const wxString &caption, float &value);

#endif /* _IUTILS_H */
