/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _LAPLACEFILTER_H
#define _LAPLACEFILTER_H

/** 
 * Implements Laplace filter.
 */
class LaplaceFilter
{
public:

	/**
	 * Constructs a LaplaceFilter.
	 * @param size		Number of input and output values.
	 * @param dimension Dimension of filter.
	 * @param laplace   Pointer to array of dimension Laplace values, NULL is default to be used.
	 *					Caution: array is deleted when filter is destructed.
	 */
	LaplaceFilter(wxUint8 size, wxUint8 dimension = 3, float *laplace = NULL);

	/**
	 * Destructor.
	 */
	~LaplaceFilter();

	/** 
	 * Apply the Laplace filter on the given input. 
	 * @param input Pointer to array containing mSize input bytes. 
	 * @return Pointer to array containing mSize output values.
	 */
	const float *Apply(const wxByte *input);

	/** 
	 * Apply the Laplace filter on the given input. 
	 * @param input Pointer to array containing mSize input floats between 0 and 1. 
	 * @return Pointer to array containing mSize output values.
	 */
	const float *Apply(const float *input);

	/** Returns number of input and output values of this filter. */
	wxUint8 GetSize() { return mSize; }
	/** Returns dimension of this filter. */
	wxUint8 GetDimension() { return mDimension; }

protected:
	wxUint8 mDimension, mSize;
	float *mOutput;  /**< Pointer to array of mSize output values. */
	float *mTemp;	 /**< Pointer to array of mSize temporary floating values. */
	float *mLaplace; /**< Pointer to array of mDimension weights used when applying filter. */
};

#endif /* _LAPLACEFILTER_H */
