/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>
#include <wx/file.h>
#include <wx/datetime.h>

#include "../status.h"
#include "iloggable.h"

#include "logger.h"

Logger::Logger(const wxString& filename, ILoggable *loggable) {
	mLoggable = loggable;
	mFile = new wxFile(filename.c_str(), wxFile::write_append);
	if(!mFile->IsOpened()) {
		StatusLine("Logger - Couldn't open file \"" + filename + "\" for writing.");
		delete mFile;
		mFile = NULL;
	} else {
		WriteString("%Log started at " + wxDateTime::Now().Format() + "\n");
		mLoggable->StartLog(this);
	}
}

Logger::~Logger() {
	if(mFile) {
		mLoggable->EndLog(this);
		WriteString("%Log ended at " + wxDateTime::Now().Format() + "\n\n");
		delete mFile;
	}
}

void Logger::Update(Observable* observable, void* argument) {
	if(mFile)
		mLoggable->Log(this);
}

bool Logger::Ok() {
	return mFile->IsOpened();
}

void Logger::WriteString(const wxString& string) {
	if(!mFile->Write(string)) {
		StatusLine("Logger - Error during write, closing log.");
		StatusLine("Logger - Tried to write \"" + string + "\"");
		delete mFile;
		mFile = NULL;
	} else
		mFile->Flush();
}
