/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _LOGGER_H
#define _LOGGER_H

class wxFile;
#include "observable.h"
#include "iloggable.h"

/**
 * An Observer that can be used to log data of ILoggable classes.
 * @see ILoggable
 */
class Logger : public IObserver
{
public:
	/**
	 * Constructor.
	 * @param filename File to write log to.
	 * @param loggable Class that is able to write log.
	 */
	Logger(const wxString& filename, ILoggable *loggable);
	
	/** Destructor. */
	virtual ~Logger();
	
	/** Called when new data is to be written to log. */
	void Update(Observable* observable, void* argument);
	
	/** Returns true if logger is ok. */
	bool Ok();
	
	/** Should be called by ILoggables to write to log. */
	void WriteString(const wxString& string);
	
protected:
	ILoggable *mLoggable;
	wxFile *mFile;
};

#endif /* _LOGGER_H */
