/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>

#include "observable.h"

Observable::Observable() {}
Observable::~Observable() {}

void Observable::AddObserver(IObserver *observer) {
	mObservers.Add(observer);
}

int Observable::CountObservers() {
	return mObservers.Count();
}

void Observable::DeleteObserver(IObserver *observer) {
	mObservers.Remove(observer);
}

void Observable::DeleteObservers() {
	mObservers.Empty();
}

void Observable::NotifyObservers() {
	if(!mChanged) return;
	wxUint8 count = mObservers.Count();
	for(int i=0; i<count; i++)
		mObservers.Item(i)->Update(this, NULL);
	ClearChanged();
}

void Observable::NotifyObservers(void *argument) {
	if(!mChanged) return;
	wxUint8 count = mObservers.Count();
	for(int i=0; i<count; i++)
		mObservers.Item(i)->Update(this, argument);
	ClearChanged();
}
