/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _OBSERVABLE_H
#define _OBSERVABLE_H

#include "iobserver.h"

WX_DEFINE_ARRAY(IObserver *, IObserverArray);

/**
 * Observed by an Observer.
 */
class Observable
{
public:
	/** Construct an Observable. */
	Observable();
	/** Destruct the Observable. */
	~Observable();

	/** Add an Observer to the list of Observers. */
	void AddObserver(IObserver *observer);
	/** Count the number of Observers currently observing this class. */
	int CountObservers();
	/** Delete a particular from the list. */
	void DeleteObserver(IObserver *observer);
	/** Delete all Observers from the list. */
	void DeleteObservers();
	
	/** Returns true if the state of the Observable has been changed. */
	bool HasChanged() { return mChanged; }
	/** Notify all Observers if the state has been changed. */
	void NotifyObservers();
	/** 
	 * Notify all Observers if the state has been changed.
	 * @param argument Argument to pass on to Observer.
	 */
	void NotifyObservers(void *argument);

protected:
	void SetChanged() { mChanged = true; }
	void ClearChanged() { mChanged = false; }
	bool mChanged;

	IObserverArray mObservers;
};

#endif /* _OBSERVABLE_H */
