/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <wx/wx.h>

#include "randomtools.h"

void RandomTools::Init(wxUint32 seed) {
	srand(seed);
}

bool RandomTools::Flip(float p) {
	if(rand() <= (int)(RAND_MAX * p))
		return true;
	else
		return false;
}

wxUint16 RandomTools::RandInt16(wxUint16 max) {
	return (wxUint16)((max - 1) * (rand() / (float)RAND_MAX));
}

float RandomTools::RandFloat(float a) {
	return 2.0f * a * (rand() / (float)RAND_MAX) - a;
}

double RandomTools::RandUnityDouble() {
	return rand() / (double)RAND_MAX;
}

float RandomTools::RandUnityFloat() {
	return rand() / (float) RAND_MAX;
}

float RandomTools::RandGaussian(float mean, float sigm)
{
	// Generation using the Polar (Box-Mueller) method.
	// Code inspired by GSL, which is a really great math lib.
	// http://sources.redhat.com/gsl/
	// C++ wrapper available.
	// http://gslwrap.sourceforge.net/

	float r, x, y;
	
	// Generate random number in unity circle.
	do
	{
		x = RandFloat(1);
		y = RandFloat(1);
		r = x*x + y*y;
	}
	while(r > 1.0 || r == 0);
	
	// Box-Muller transform.
	return sigm * y * sqrt (-2.0 * log(r) / r) + mean;
	
}