/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#ifndef _RANDOMTOOLS_H
#define _RANDOMTOOLS_H

/**
 * A utilities class to generate numbers randomly
 */
 
class RandomTools
{
public:
   /**
   * Static method to initialize the random number gennerator
   */
  static void Init(wxUint32 seed);

  /**
   * Static method that returns a random boolean number
   * @param p is the probability of return 1
   * @return the random boolean number   
   */
  static bool Flip(float p);

  /**
   * Static method that returns an random int between 0 and max-1 
   * @param max the upper bound of the number returned
   * @return the random int number
   */
  static wxUint16 RandInt16(wxUint16 max);


  /**
   * Static method that returns a random float between -a and +a
   * @param a the upper bound of the number returned, the lower bound is -a
   * @return the random float number
   */
  static float RandFloat(float a);


  /**
   * Static method that returns a random double between 0.0 and 1.0
   * @return the random double number
   */
  static double RandUnityDouble();
  
  /** Returns a random float between 0.0f and 1.0f. */
  static float RandUnityFloat();
  
  /** Retruns a random float using a gaussian distribution. */
  static float RandGaussian(float mean, float sigm);
};

#endif /*_RANDOMTOOLS_H */
