/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/


#include <wx/wx.h>
#include <wx/sckipc.h>

#include "smtp.h"

wxString Smtp::mSmtpHost = "localhost";
wxUint16 Smtp::mSmtpPort = 25;

wxInt8 Smtp::SendMail(const wxString &from, const wxString &to, const wxString &subject, const wxString &body) {
	wxIPV4address addr;

	addr.Hostname(mSmtpHost);
	addr.Service(mSmtpPort);
	
	wxSocketClient *socketClient = new wxSocketClient();
	
	socketClient->Connect(addr, false);
	socketClient->WaitOnConnect(5, 1000);
	if(!socketClient->IsConnected())
		return 1;
	
	wxString buffer;
	ReadLine(socketClient, buffer);
	buffer = "HELO i\n";
	if(WriteLine(socketClient, buffer)) return 2;
	if(ReadLine(socketClient, buffer)) return 3;
	buffer = "MAIL FROM:" + from + "\n";
	if(WriteLine(socketClient, buffer)) return 2;
	if(ReadLine(socketClient, buffer)) return 3;
	buffer = "RCPT TO:" + to + "\n";
	if(WriteLine(socketClient, buffer)) return 2;
	if(ReadLine(socketClient, buffer)) return 3;
	buffer = "DATA\r\n";
	if(WriteLine(socketClient, buffer)) return 2;
	if(ReadLine(socketClient, buffer)) return 3;
	buffer = "From: " + from + "\r\nTo: " + to + "\r\nSubject: " + subject + "\r\n\r\n" + body + "\r\n.\r\n";
	if(WriteLine(socketClient, buffer)) return 2;
	if(ReadLine(socketClient, buffer)) return 3;
	buffer = "QUIT\n";
	if(WriteLine(socketClient, buffer)) return 2;
	if(ReadLine(socketClient, buffer)) return 3;
	socketClient->Destroy();
	return 0;
}

wxInt8 Smtp::WriteLine(wxSocketClient *socket, wxString &line) {
	socket->Write(line.c_str(), line.Len());
	if(socket->Error())
		return 1;
	return 0;
}

wxInt8 Smtp::ReadLine(wxSocketClient *socket, wxString &result) {
    result.Empty();
    char ch, chLast = '\0';
    while ( !socket->Read(&ch, sizeof(ch)).Error() )
    {
        switch ( ch )
        {
            case '\r':
                // remember it, if the following is '\n', we're done
                chLast = '\r';
                break;

            case '\n':
                // only ends line if the previous character was '\r'
                if ( chLast == '\r' )
                {
                    // EOL found
                    return 0;
                }
                //else: fall through

            default:
                // normal char
                if ( chLast )
                {
                    result += wxString::FromAscii( chLast );
                    chLast = '\0';
                }

                result += wxString::FromAscii( ch );
        }
    }

    return 1;
}
