/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

#include "wx/wxprec.h"
#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include "xmlattribute.h"
#include "xmlutils.h"

XMLAttribute::XMLAttribute (const wxString &name, const wxString &value, const bool encode) {
	InitWith(name,value,encode);
}

XMLAttribute::XMLAttribute(const wxString &name, const wxInt32 value) {
	wxString s = "";
	s<<value;
	InitWith(name,s,false);
}

XMLAttribute::XMLAttribute(const wxString &name, const float value) {
	wxString s = "";
	s<<value;
	InitWith(name,s,false);
}


XMLAttribute::XMLAttribute(const wxString &name, const double value){ 
	wxString s = "";
	s<<value;
	InitWith(name,s,false);
}


/*XMLAttribute::XMLAttribute(const wxString &name, const bool value) {
	wxString s = "";
	if(value) {
		s<<"true";
		InitWith(name,s,false);
	} else {
		s<<"false";
		InitWith(name,s,false);
	}
}*/

XMLAttribute::XMLAttribute(const wxString &name, const wxByte value) {
	wxString s = "";
	s<<value;
	InitWith(name,s,false);
}

void XMLAttribute::InitWith(const wxString &name, const wxString &value, const bool encode) {
	if (encode) {
		wxString *pName = new wxString(name);
		wxString xName = XMLUtils::ToXMLString(*pName);
		this->name = new wxString(xName);
		delete pName;

		wxString *pValue = new wxString(value);
		wxString xValue = XMLUtils::ToXMLString(*pValue);
		this->value = new wxString(xValue);
		delete pValue;
    } else { 
		this->name = new wxString(name);
		this->value = new wxString(value);
    }
}

XMLAttribute::~XMLAttribute() {
	delete name;
	delete value;
}


wxString XMLAttribute::ToString () {
	return *name+'='+'\"'+*value+'\"';
}

wxString XMLAttribute::GetName () {
	return XMLUtils::FromXMLString(*name);
}

wxString XMLAttribute::GetValue () {
	return XMLUtils::FromXMLString(*value);
}

wxInt32 XMLAttribute::GetIntValue() {
	long longvalue;
	value->ToLong(&longvalue);
	return(longvalue);
}

long XMLAttribute::GetLongValue() {
	long long_value;
	value->ToLong(&long_value);
	return(long_value);
}

double XMLAttribute::GetDoubleValue() {
	double double_value;
	value->ToDouble(&double_value);
	return(double_value);
}

bool XMLAttribute::GetBoolValue() {
	if (value->IsSameAs("true") || value->IsSameAs("1")) return true;
    if (value->IsSameAs("false") || value->IsSameAs("0")) return false;
	else {
		//error message	
		return false;
	}
//wxString XMLAttribute::GetStringValue() {
//	return(value);
//}

}

float XMLAttribute::GetFloatValue() {
	double double_value;
	float float_value;
	value->ToDouble(&double_value);
	float_value = (float)double_value;
	return float_value;
}  

wxByte XMLAttribute::GetByteValue() {
	unsigned long long_value;
	wxByte byte_value;
	value->ToULong(&long_value);
	byte_value = (wxByte) long_value;
	return byte_value;
}  
