/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

#ifndef _XMLATTRIBUTE_H
#define _XMLATTRIBUTE_H

class XMLAttribute {
public:
	/** Make a new XMLAttribute. Choose if you want to encapsulate control chars.
		* @param name The name of the attribute
		* @param value The value of the attribute
		* @param encode If this boolean is true, then control characters in
		*        the parameters will be encapsulated so they don't confuse
		*        our XML parser.
	*/
	XMLAttribute(const wxString &name, const wxString &value, const bool encode=true);
  
	/** Make a new XMLAttribute. The value is an int 
	  * @param name The name of the attribute
	  * @param value The value of the attribute
	 */
	XMLAttribute(const wxString &name, const wxInt32 value);

	/** Make a new XMLAttribute. The value is a float
	  * @param name The name of the attribute
	  * @param value The value of the attribute
	 */
	XMLAttribute(const wxString &name, const float value);

	/** Make a new XMLAttribute. The value is a double
	  * @param name The name of the attribute
	  * @param value The value of the attribute
	 */
	XMLAttribute(const wxString &name, const double value);
  
	/** Make a new XMLAttribute. The value is a boolean
	  * @param name The name of the attribute
	  * @param value The value of the attribute
	 */
	/*XMLAttribute(const wxString &name, const bool value);*/
  
	/** Make a new XMLAttribute. The value is a byte
	  * @param name The name of the attribute
	  * @param value The value of the attribute
	 */
	XMLAttribute(const wxString &name, const wxByte value);

	~XMLAttribute();
  
	void		InitWith(const wxString &name, const wxString &value, const bool encode);

	/** @return A string with name="value". With eventual encapsulation. */
	wxString	ToString();
  
	/* you delete */
	/** @return The name of this attribute	*/
	wxString	GetName();

	/* you delete */
  	/** @return The value of this attribute	*/
	wxString	GetValue();
  
	/** @return Tries to convert the value of this attribute to an int and returns it */
	wxInt32 GetIntValue();
  
	/** @return Tries to convert the value of this attribute to an float and returns it */
	long GetLongValue();

	/** @return Tries to convert the value of this attribute to a boolean and returns it */
	bool GetBoolValue();

	/** @return Tries to convert the value of this attribute to a float and returns it */
	float GetFloatValue();

	/** @return Tries to convert the value of this attribute to a double and returns it	*/
	double GetDoubleValue();

	/** @return Tries to convert the value of this attribute to a byte and returns it */
	wxByte GetByteValue();

	//wxString GetStringValue()

protected:
	wxString *name, *value;
};
#endif
