/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

#ifndef _XMLELEMENT_H
#define _XMLELEMENT_H

//#include "xmlattribute.h"
#include <wx/dynarray.h>
#include "xmlarray.h"

class XMLAttribute;

class XMLElement {
public:

	/** Make a new XMLElement
	  * @param name The name of the new element
	 */
	XMLElement (const wxString &name);


	/** Make a new XMLElement with the specified name and attributes
	  * @param name The name of the new element
	  * @param attList An array that contains the attributes for this
	  *                element (instances of gld.xml.XMLAttribute)
	 */
	XMLElement (const wxString &name, XMLAttributeArray* attList);

	~XMLElement();

	  
	/** @return The name of this XML element */
	wxString		GetName ();

	/** Change the name of this XML element
	  * @param name The new name
	 */
	void			SetName(const wxString &name);

	  
	/** Add a new attribute to the attribute list
		* @param attribute The new attribute
	*/
	void AddAttribute(XMLAttribute* attribute);

	  
	/** Remove the (first) attribute with this name
		* @param name The name of the attribute which has to be removed
		* @throws NoSuchElementException If an attribute with that name
		*         cannot be found.
	*/
	void RemoveAttribute(const wxString &name);

	  
	/** Get the (first) attribute with a certain name
	  * @param name The name to search for
	  * @throws NoSuchElementException If an attribute with that name cannot be found.
	  * @return The found attribute
	 */
	XMLAttribute*	GetAttribute(const wxString &name);

	  
	/* @return A string which describes this XMLElement	*/
	wxString		ToString ();


	/* @return An object copy of this XML element
	Object clone ();
	*/
	  
	/** Reset the attribute list (make it empty) */
	void			RemoveAllAttributes();

	  
	/** @return The attribute list in array form */
	XMLAttributeArray* GetAttributesArray();

	  
	/** Change the attribute with the name of the parameter attribute to the
	  * value of the parameter attribute. If an attribute with the name of the
	  * parameter doesn't exist, then add the parameter attribute to the
	  * attribute list.
	  * @param attribute The attribute to set.
	 */
	void			SetAttribute(XMLAttribute* attribute);
	 
	  
	/** @return a string which represents the XML tag which opens this element in the XML file */
	wxString* GetOpenTag ();

	  
	/** @return Returns a string which represents the XML tag which closes this
	  *         element in the XML file.
	 */
	wxString* GetCloseTag();

	  
	/** Parse a new XML element from a string which contains a XML tag
	  * @param string The string to parse
	 */
	static XMLElement* Parse(const wxString &string);

protected:
	wxString			*mName;
	XMLAttributeArray	*mAttributes;

	/** Get the index of an attribute in the attribute array
	* @param name The name to search for
	* @throws NoSuchElementIndex If an attribute with that name cannot be found.
	* @return The index of the attribute
	*/
	wxInt32 GetAttributeIndex (const wxString &name);
};

#endif
