/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

#include "wx/wxprec.h"
#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include "../../status.h"

#include "xmlutils.h"
#include "xmlstack.h"
#include "xmlelement.h"
#include "xmlattribute.h"
#include "xmlfilereader.h"

#include "xmlloader.h"

/** This class can restore the state of the program from an XML file. It also
  * provides methods which XMLSerializable objects can use to retrieve
  * their child objects.
 */

XMLLoader::XMLLoader(const wxString &file) {
	mStack = new XMLStack();
	mReader = new XMLFileReader(file);
	mNextElement = NULL;
	LookAhead();
}

XMLLoader::~XMLLoader() {
	delete mStack; mStack = NULL;
	delete mNextElement; mNextElement = NULL;
	delete mReader; mReader = NULL;
}

void XMLLoader::LookAhead () {
	wxString tag;
    // Read an open tag , eventually preceded by closing tags
    while ( mNextElement==NULL && !mReader->IsDataLeft() ) { // Read tag 
		tag = mReader->ReadNextTag();
		StatusLine(tag);

		if(!tag.StartsWith("</")) 
			mNextElement = XMLElement::Parse(tag);
		else
			tag = mStack->Pop();
	}

    // Read an open tag , eventually preceded by closing tags
    /*while ( mNextElement==NULL && !mAtEOF ) { // Read tag 
		tag = mReader->ReadNextTag();
			//if (! stack.isEmpty() )		throw new XMLTreeException ("EOF at non-terminated tree. File is probably truncated.");
			//else						return; // End of tree. We don't have to look ahead.	 

		// Process Closing tags
		if (tag.StartsWith ("</")) {
			ref = mStack->Pop();
			delete &ref;
			tagged = tag.Mid(2);
			delete &tag; tag = tagged;
			//	if (!tag->IsSameAs(ref->c_str())) 	("Unbalanced tags while loading XML file :"+ref+"/"+tag);
		}
		// And begin tag
		else
			mNextElement = XMLElement::Parse(tag);
		//delete tag; tag=NULL;
	}*/


}

XMLElement* XMLLoader::PullNextElement() {
	XMLElement *result = mNextElement;

    mNextElement=NULL;

    mStack->Push(result->GetName());
    LookAhead();
    return result;
}


void XMLLoader::Load (XMLSerializable *parent, XMLSerializable *object) {
	XMLElement *elem = GetNextElement(parent, XMLUtils::GetLastName(object));
	object->Load(elem, this);
	delete elem;
}


void XMLLoader::Close() {
	delete mReader;
	mReader = NULL;
}
  
void XMLLoader::LoadEnumeration (XMLSerializable *parent, XMLSerializableArray *a) {
	for(wxUint32 i=0; i<a->GetCount(); i++) 
		Load(parent,a->Item(i));
}
  
XMLElement* XMLLoader::GetNextElement(XMLSerializable *parent) {
	/* stack.assertIsCurrentParent(parent); */
	if (mNextElement==NULL) {
		/* throw new XMLTreeException ("Can't find more XMLElements inside "+parent.getXMLName()); */
		return NULL;
    }
    return PullNextElement();
}
  
bool XMLLoader::HasNextElement(XMLSerializable *parent) {
	return mStack->IsCurrentParent(parent) && mNextElement!=NULL;
}
  
XMLElement* XMLLoader::GetNextElement(XMLSerializable *parent, const wxString &name) {
	/*wxASSERT_MSG(mStack->IsCurrentParent(parent), "Current parent is not the current parent!");*/
	if(mNextElement==NULL)
		return NULL;
	/*wxASSERT_MSG(mNextElement!=NULL,
		"Can't find more XMLElements inside "+parent->GetXMLName());*/
	if(!(!name.IsEmpty() && name.IsSameAs(mNextElement->GetName().c_str())))
		return NULL;
	/*wxASSERT_MSG(!name.IsEmpty() && name.IsSameAs(mNextElement->GetName().c_str()),
		"Can't find next element named "+name+" in parent "+ XMLUtils::GetLastName(parent->GetXMLName()));*/
    return PullNextElement();
}

bool XMLLoader::HasNextElement (XMLSerializable *parent, const wxString &name) {
	return HasNextElement(parent) && mNextElement->GetName().IsSameAs(name.c_str());
}

/* you -dont- delete */
wxString XMLLoader::GetNextElementName () {
	if (mNextElement==NULL)
		return "";
    else
		return mNextElement->GetName();
}
