/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

/** A utility class to read XML from an InputStream */

#include "wx/wxprec.h"
#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include <wx/stream.h>
#include "xmlreader.h"

XMLReader::XMLReader(wxInputStream *stream) {
	mStream = stream;
    mBuffer = new char[mBufferSize];

	mAtEOF = false;
	mBufferEnd = 0;
    mBufferBegin = 0;
	mStreamIndex = 0;
}

XMLReader::~XMLReader() {
	delete mBuffer;
	delete mStream;
}
  
bool XMLReader::IsAtEOF() {
	return mAtEOF;
}

bool XMLReader::IsDataLeft() {
	return mAtEOF && mBufferBegin==mBufferEnd;
}

void XMLReader::FillBuffer() {
	if (BufferIsEmpty()) {
		mBufferBegin=0;
		mStream->Read(mBuffer, mBufferSize);
		mBufferEnd = mStream->TellI() - mStreamIndex;
		mStreamIndex = mStream->TellI();
		if(mStream->Eof())
			mAtEOF=true;
	}
}

wxString XMLReader::ReadNextTag() {
	IgnoreUntilChar('<',false);
	wxString thisTag = ReadUntilChar ('>',true);
    return thisTag;
}
  
bool XMLReader::BufferIsEmpty () {
	return mBufferBegin==mBufferEnd;
}
  
wxString XMLReader::Readln () {
	wxString result = ReadUntilChar ('\n',false);
    mBufferBegin++;
    return result;
}

wxString XMLReader::ReadUntilChar (const wxChar &c, const bool including) {
	wxString result = "";
	wxString *tmp = NULL;
	wxUint32 t;
	while (true) {
		FillBuffer();
		for (t=mBufferBegin; t<mBufferEnd; t++) {
			if (c==mBuffer[t]) {
				wxInt32 endPos=t-mBufferBegin;
                if (including)
					endPos++;
				tmp = new wxString(mBuffer,mBufferBegin,endPos);
				result += *tmp;
				delete tmp;
				mBufferBegin=t; 
				if(including)
					mBufferBegin++;
				return result;
			}
		}
		tmp = new wxString(mBuffer,mBufferBegin,mBufferEnd-mBufferBegin);
		result += tmp->c_str();
		delete tmp;
		mBufferBegin=mBufferEnd;
		// This statement is superfluous, just to make really sure the
		// loop is eventually terminated if the char is never encountered.
		if (mAtEOF) {
			return "";
		}
	}
	delete tmp;
}
  
void XMLReader::IgnoreUntilChar (const wxChar &c, const bool including) {
	while (!mAtEOF || mBufferBegin!=mBufferEnd) {
		FillBuffer();
        // Check the first char in the buffer
        if (mBufferBegin<mBufferEnd && mBuffer[mBufferBegin]==c) {
			if (including)
				mBufferBegin++;
			return;
		}
		for ( ; mBufferBegin<mBufferEnd; mBufferBegin++) {
			if (mBuffer[mBufferBegin]==c) {
				if (including)
					mBufferBegin++;
				return;  
			}
		}
	}
}
