/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

#ifndef _XMLREADER_H
#define _XMLREADER_H

class wxFileInputStream;

class XMLReader
{
public:
	/** Make a new XMLReader
	  * @param stream The InputStream on which this XMLReader is based
	*/
	XMLReader(wxInputStream *stream);

	~XMLReader();

	/** Fill the internal buffer of this Reader */
	void		FillBuffer();

	/** Read next XML tag
	  * @return The tag which is read
	 */
	wxString	ReadNextTag();

	/* @return A boolean which indicates if the buffer is empty  */
	bool		BufferIsEmpty();

	/** Read a string until we encounter a specific character.
	  * @param c The specific char
	  * @param including True if the char should be included in the string,
	  *        false if it should be discarded
	  * @return The result	
	 */
	wxString	ReadUntilChar (const wxChar &c, const bool including); 

	/** Move the buffer to the first occurence of a certain char.
	  * @param c The specific char
	  * @param including True if the buffer should be moved to one position 
	  *        after the char, false if it should be positioned at the char itself.
	 */
	void		IgnoreUntilChar (const wxChar &c, const bool including);

	bool		IsDataLeft();

protected:
	/** Read a line 
	  * @return A string which contains the next line of the file
	 */
	wxString	Readln();
	bool		IsAtEOF();

	wxInputStream			*mStream;
	static const wxUint32	mBufferSize=1024;
	wxUint32				mBufferBegin, mBufferEnd, mStreamIndex;
	bool					mAtEOF;
	char					*mBuffer;
};

#endif
