/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

#include "wx/wxprec.h"
#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include "../../status.h"

#include "xmlutils.h"
#include "xmlsaver.h"
#include "xmlstack.h"
#include "xmlelement.h"
#include "xmlfilereader.h"
#include "xmlfilewriter.h"
#include "xmlserializable.h"

/** This class takes care of saving the state of the program to a XML
  * file. It also provides methods which XMLSerializable objects can
  * use to save their child objects.
 */

/** Make a new XMLSaver  */
XMLSaver::XMLSaver() {
	mStack = new XMLStack();
}

/** Makes a new XMLSaver which saves to a file
  * @param file The file to save to
 */
XMLSaver::XMLSaver (const wxString &file) {
	mStack = new XMLStack();
    mWriter = new XMLFileWriter(file);
}

XMLSaver::~XMLSaver() {
	delete mStack;
	delete mWriter;
	mStack = NULL;
	mWriter = NULL;
}

/** A crude indication if we can write to this XMLSaver */
bool XMLSaver::HasStream() {
	return mWriter != NULL;
}
  
/** Close this XMLSaver */
void XMLSaver::Close () {
	/*writer->flush();
	writer->close();*/
	delete mWriter;
	mWriter=NULL;
}

/** Save a XMLSerializable object to the XML file
  * @param object The object to save
 */
void XMLSaver::SaveObject (XMLSerializable *object) {
	//Initialize various names of the object
	wxString fullName	= object->GetXMLName();
	wxString parentName	= XMLUtils::GetParentName(object);
	wxString firstName	= XMLUtils::GetLastName(object);
    
	// Check if the object can be saved.
	//ASSERT(stack->IsCurrentParent(parentName));

	// Save element of object
	StatusLine(firstName);
	mStack->Push(firstName);

	XMLElement *element = object->SaveSelf();
	mWriter->WriteOpenTag(element,Indent());

	// Let object save its child objects
	object->SaveChilds(this);

	// Finish element
	mWriter->WriteCloseTag(element,Indent());
	mStack->Pop();

	delete element;
	
	StatusLine("done");	

	
	// Check if the stack is OK
	//if ((!mStack->GetBranchName()->IsSameAs(parentName->c_str())) &&
	//	(!XMLUtils::GetGenericName(mStack->GetBranchName())->IsSameAs((XMLUtils::GetGenericName(parentName))->c_str()))) {
		/*
			als dit gebeurd, is er Boel Fout:
			throw new XMLTreeException
      ("The XMLSerializable "+fullName+
       " (or possibly one of its child objects) left the XML tree "+
       "in an invalid state. Terminating save.\n"+
       "Expected branchName :"+stack.getBranchName()+"/"+
                               XMLUtils.getGenericName(stack.getBranchName())+"\n"+
       "but branchname was  :"+parentName+"/"+
                               XMLUtils.getGenericName(parentName));
		*/
    //}
}

/** Save an enumeration of XML Serializables to file 
  * @param e The enumeration of XML Serializables
 */
void XMLSaver::SaveObjectArray (XMLSerializableArray *a) {
	for(wxUint32 i=0; i<a->GetCount(); i++)
		SaveObject((XMLSerializable*)(a->Item(i)));
}
  
/** Save an atomary XML element
  * @param el The XMLElement to save
 */
void XMLSaver::SaveAtomaryElement (XMLSerializable *parent, XMLElement *el) {
	//stack->AssertIsCurrentParent(parent);
 	mWriter->WriteAtomaryElement(el,Indent());
}
  
/** @return An internal indentation string */
wxInt32 XMLSaver::Indent () {
	return mStack->Size();
}
