/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

#ifndef _XMLSERIALIZABLE_H
#define _XMLSERIALIZABLE_H
/** This interface has to be implemented by objects that want to 
  * be loaded and saved via the parser.
  */

class XMLElement;
class XMLLoader;
class XMLSaver;
class wxString;

class XMLSerializable { 
public:
 /** Load this XMLSerializable
  *  @param myself The XMLElement which represents this object in the
  *         XML tree. It can contain attributes which hold information
  *         about the object.
  *  @param loader The XMLLoader which this XMLSerializable can use to
  *         load child objects.
  */
 virtual void Load (XMLElement *myself, XMLLoader *loader)=0;

 /** @return The XMLElement which represents the internal datastate. This
  *          function is not meant to save child objects. That is done in
  *          void saveChilds()
  */
 virtual XMLElement* SaveSelf ()=0;
  
 /** This method gives an XMLSerializable the opportunity to save 
  *  its child objects.
  *  @param saver The XMLSaver that the XMLSerializable can use to save its
  *         child objects.
  *  @throws XMLTreeException The parser can throw this exception if it
  *          is called by the XMLSerializable. The XMLSerializable should NOT
  *          throw this exception by itself and preferrably also not catch it too.
  *  @throws IOException Thrown in case the parser cannot write to the file
  *  @throws XMLCannotSaveException The XMLSerializable can throw this
  *          exception if it cannot save itself or one of its child objects for
  *          whatever reason.
  */
 virtual void		SaveChilds (XMLSaver *saver)=0;

 /** @return The tagname of the XML element in which information about
  *         this object is stored prepended by a dot separated list 
  *         of the tagnames of the parents of this object. (e.g. 
  *         "model.infrastructure.node")
  */
 virtual wxString	GetXMLName ()=0;
 
 /** Sets a new parent name for this object
   * @param parentName The new parent name
   * @throws XMLTreeException If this object does not support setting other
   *         parent names.
  */
 virtual void		SetParentName (const wxString &parentName)=0;
};
#endif
