/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

/** This class contains a few static utility methods that are
  * used by the various parts of the XML parser
 */
#include "wx/wxprec.h"
#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include <math.h>
#include <wx/dynarray.h>
#include "xmlutils.h"
#include "xmlserializable.h"

wxString XMLUtils::GetLastName (const wxString &fullName) {
	return fullName.AfterLast('.');
}

wxString XMLUtils::GetLastName(XMLSerializable *object) {
	wxString bla = object->GetXMLName();
	return GetLastName(bla);
}

wxString XMLUtils::GetParentName (const wxString &fullName) {
	wxInt8 lastDotPos = fullName.Find('.', true);
    if (lastDotPos==-1)
		return "";
    else
		return fullName.Mid(0,lastDotPos);
}

wxString XMLUtils::GetParentName (XMLSerializable *object) {
	return GetParentName(object->GetXMLName());
}

wxInt32 XMLUtils::GetLevel (const wxString &fullName) {
	return fullName.Freq('.');
}


wxString XMLUtils::ToXMLString (const wxString &s) {
	wxString xmld = s;
	if (xmld.IsSameAs("")) {
		return "&null;";
	}
    const char *from[] = {"/","<",">","\""," ","&"};;
    const char *to[] = {"&amp;","&#32;","&quot;","&lt;","&gt;","&#47;"};
	for(wxInt8 i=0; i<6; i++)
		xmld.Replace(from[i],to[i]);
    return xmld;
}

wxString XMLUtils::FromXMLString (const wxString &s) {
	wxString xmld = s;
	if(xmld.StartsWith("&null;")) {
		return "";
	}
	const char *from[] = {"&#47;","&lt;","&gt;","&quot;","&#32;","&#amp"};
	const char *to[]   = {"/","<",">","\""," ","&"};
	for(wxInt8 i=0; i<6; i++)
		xmld.Replace(from[i],to[i]);
    return xmld;
} 

/* you delete */
wxString* XMLUtils::GetGenericName(const wxString &fullName) {
	return GetGenericName(fullName,true);
}

/* you delete */
wxString* XMLUtils::GetGenericName(const wxString &fullName, bool removeArrays) {
	wxString tmp1 = fullName;
	wxString tmp2;
	wxString result = "";
	result.Alloc(tmp1.Len());

    	wxInt16 dashIndex;
    	while (tmp1.Find('.')!=-1) {
		tmp2 = tmp1.BeforeFirst('.');	// Current Token
		tmp1 = tmp1.AfterFirst('.');		// Rest of wxString
		if ( removeArrays && (tmp2.IsSameAs("array") || tmp2.IsSameAs("element")))
			continue;
		if (result.Len() > 0)
			result.Append(".");
		if ((dashIndex = tmp2.Find('-')) == -1)
			result.Append(tmp2);
		else
			result.Append(tmp2.Mid(0,dashIndex));
	}
	result.Shrink();
	return new wxString(result);
}

wxString XMLUtils::GetSpecificLastName(const wxString &fullName) {
	wxUint16 val1 = fullName.Find('.',true), val2 = fullName.Find('-',true);
	wxUint16 beginIndex = val1 > val2 ? val1 : val2;
	beginIndex = beginIndex > 0 ? beginIndex : 0;
	beginIndex++;
    	return fullName.Mid(beginIndex);
}
  
void XMLUtils::SetParentName(XMLSerializableArray *a, const wxString &newParentName) {
	for(wxUint32 i=0; i<a->GetCount(); i++)
		((XMLSerializable*)(a->Item(i)))->SetParentName(newParentName);
}
