/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

#ifndef _XMLUTILS_H
#define _XMLUTILS_H
/** This class contains a few static utility methods that are
  * used by the various parts of the XML parser
 */
#include <wx/dynarray.h>
#include "xmlarray.h"

class XMLSerializable;

class XMLUtils {
public:
	/** Calculates the last name of a XML path name
	  * @param fullName The path name
	  * @return Its last name
	 */
	static wxString		GetLastName (const wxString &fullName);

	/** Calculates the last name of a XMLSerializable object
	  * @param object The XMLSerializable object
	  * @return Its last name
	 */
	static wxString		GetLastName(XMLSerializable *object);
	
	/** Takes the last name of a full XML tag name. If it consists of a generic
	  * part and a specific part, then the generic part is removed.
	  * examples : "model.infrastructure.node-edge" becomes "edge"
	  *         :  "model.infrastructure' becomes "infrastructure"
	  * @param fullName The full XML tag name to convert
	  * @return The result
	 */	
	static wxString		GetSpecificLastName(const wxString &fullName);

	/** Calculates the parent path name of a XML path name
	  * @param fullName The path name
	  * @return The path of its parent
	 */
	static wxString		GetParentName (const wxString &fullName);

	/** Calculates the parent path name of a XML path name
	 * @param object The XMLSerializable object
	 * @return The path of its parent
	*/
	static wxString		GetParentName (XMLSerializable *object);

	/** Sets new parentNames for every element in an enumeration of
	  * XMLSerializables.
	  * @param e The enumeration of XMLSerializables
	  * @param newParentName The new p
	  * @throws ClassCastException If one of the elements in the enumeration
	  *         is not XMLSerializable
	 */
	static void			SetParentName(XMLSerializableArray *a, const wxString &newParentName);

	/** Encodes characters which can confuse the XML parser. Every 
	  * non-alphanumeric string should be processed by this method before saving!
	  * @param s The string to encode
	  * @return The result
	 */
	static wxString		ToXMLString (const wxString &s);

	/** Decodes a string which is encoded with toXMLString. 
	  * @param s The string to decode
	  * @return The result
	 */
	static wxString		FromXMLString (const wxString &s);

	/** Convert a full XML name to its generic name. This is done by checking
	  * if each part of the name is complex (if it consists of a generic name plus
	  * a specific name). If so, then the specific part is removed. Array elements
	  * are also removed from the name.
	  * Example of a complex name : model.infrastructure.node-edge.spdata
	  * Result of getGenericName  : model.infrastructure.node.spdata
	  * @param name The XML name
	  * @return The result
	 */
	static wxString*		GetGenericName(const wxString &fullName);

	/** Convert a full XML name to its generic name. This is done by checking
	  * if each part of the name is complex (if it consists of a generic name plus
	  * a specific name). If so, then the specific part is removed. 
	  * Example of a complex name : model.infrastructure.node-edge.spdata
	  * Result of getGenericName  : model.infrastructure.node.spdata
	  * @param name The XML name
	  * @param removeArrays Whether array elements have to be removed from
	  *        the name.
	  * @return The result
	 */
	static wxString* 	GetGenericName(const wxString &fullName, bool removeArrays);

	/** Calculates the level of a XML path name
	  * @param fullName The path name
	  * @return the level
	 */
	static wxInt32		GetLevel (const wxString &fullName);
};
#endif
