/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

/** A utility class to write XML to an InputStream */

#include "wx/wxprec.h"
#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#include <wx/stream.h>
#include "xmlwriter.h"
#include "xmlelement.h"

/** A utility class to write XML to an OutputStream */
XMLWriter::XMLWriter(wxOutputStream *output_stream) {
	mStream = output_stream;
}

XMLWriter::~XMLWriter() {
	delete mStream;
}

void XMLWriter::WriteOpenTag (XMLElement *element, const wxUint32 indent) { 	
	wxString *string = element->GetOpenTag();
	wxUint32 i=0, j=0;
	wxUint32 str_len = string->Len();
	char *buf = new char[4096];
	for(; i<indent; i++) 
		buf[i] = ' ';
	for(; j<str_len; j++)
		buf[i+j] = string->GetChar(j);
	mStream->Write(buf,i+j);
	delete string;
	delete buf;
}

void XMLWriter::WriteOpenTag (XMLElement *element) {
	WriteOpenTag(element,0);
}

void XMLWriter::WriteCloseTag (XMLElement *element, const wxUint32 indent) { 	
	wxString *string = element->GetCloseTag();
	wxUint32 i=0, j=0, str_len=string->Length();
	char *buf;
	buf = new char[4096];
	for(; i<indent; i++) {*(buf+i)=' '; }
	for(; j<str_len; j++) {*(buf+i+j)=string->GetChar(j); }
	mStream->Write(buf,i+j);
	delete string;
	delete buf;
}

void XMLWriter::WriteCloseTag (XMLElement *element) {
	WriteCloseTag(element,0);
}


void XMLWriter::WriteAtomaryElement (XMLElement *element, const wxUint32 indent=0) { 	
	wxString *open_string = element->GetOpenTag();
	wxString *close_string = element->GetCloseTag();
	wxUint32 i=0, j=0, k=0;
	wxUint32 openstr_len = open_string->Length();
	wxUint32 closestr_len = close_string->Length();

	char *buf;
	buf = new char[4096];
	for(; i<indent; i++) {*(buf+i)=' '; }
	for(; j<openstr_len; j++) {*(buf+i+j)=open_string->GetChar(j); }
	*(buf+i+j)=' ';
	for(; k<closestr_len; k++) {*(buf+i+j+1+k)=close_string->GetChar(j); }
	delete open_string; delete close_string;
	mStream->Write(buf,i+j+1+k);
	delete buf;
}
