/****************************************************************************
  goevo -- application for evolutionary robotics with neural controllers
  Copyright (C) 2003 Autonomous Systems Lab 2, EPFL, Lausanne &
  Adaptive Intelligence Lab, University Utrecht.

  This program is free software; the authors of any publication 
  arising from research using this software are asked to add the 
  following reference:
    goevo; http://asl.epfl.ch/resources/evo/goevo.php; 
    Autonomous Systems Lab 2, EPFL, Lausanne & 
    Adaptive Intelligence Lab, University Utrecht.

  You can redistribute this program and/or modify it under the 
  terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, 
  or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  See docs/readme.txt and docs/gpl.txt
******************************************************************************/

#ifndef _XMLWRITER_H
#define _XMLWRITER_H

class XMLElement;
class wxOutputStream;

class XMLWriter { 	
public:
	/** Make a new XMLWriter
	  * @param stream The OutputStream on which this XMLWriter is based
	 */
	XMLWriter::XMLWriter (wxOutputStream *output_stream);

	XMLWriter::~XMLWriter();

	/** Write the open tag of a XMLElement 
	  * @param element The XMLElement
	  * @param indent The number of spaces to indent the tag
	 */
	void WriteOpenTag (XMLElement *element, const wxUint32 indent);

	/** Write the open tag of a XMLElement 
	  * @param element The XMLElement
	 */
	void WriteOpenTag (XMLElement *element);

	/** Write the close tag of a XMLElement 
	  * @param element The XMLElement
	  * @param indent The number of spaces to indent the tag
	 */
	void WriteCloseTag (XMLElement *element, const wxUint32 indent);

	/** Write the close tag of a XMLElement 
	  * @param element The XMLElement
	 */
	void WriteCloseTag (XMLElement *element);

	/** Write both open and close tags of a XMLElement 
	  * @param element The XMLElement
	  * @param indent The number of spaces to indent the tags
	 */
	void WriteAtomaryElement (XMLElement *element, const wxUint32 indent);

	/** Write both open and close tags of a XMLElement 
	  * @param element The XMLElement
	 */
	void WriteAtomaryElement (XMLElement *element);

protected:
	wxOutputStream		*mStream;
};
#endif
